/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Version: 0.5.1 Beta
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
  Entorno de Desarrollo Modular y Abierto
  (c) David Martnez Oliveira
  Versin Beta WIN32
  15 de Octubre de 1997
  
  ENlace dinmico de EDMA
  
  REVISIONES:-------------------------------------------------------------
  * September, 12th,2001
  * Added new primitives to manage static methods, casting and clonning
  * --------------------------------------------------------------
  * October, 14th,2001
  * Added EBufferRealloc and PRealloc function
  * ------------------------------------------------------------------
  * November, 24th, 2001
  * Update and code cleanup
*/

#ifndef EDMA_H
#define EDMA_H

#include <stdio.h>
#include "portable.h"

#ifdef __cplusplus
extern "C"{
#endif

  /*
  ** Class State Constants
  */
#define  CLASS_FREE    0     /* Free Entry */
#define  CLASS_DEF     1     /* Class is defined but not loaded */
#define  CLASS_ILOADED 2     /* Class is defined and interface loaded */
#define  CLASS_LOADED  3     /* Class is full in memory */
  
  /*
  ** Object State Constants
  */

#define OBJ_FREE        0
#define OBJ_EXIST       1
#define VIRTUAL_OBJECT	2

  /*
  ** Machines *************************************
  */

#define MAQ_NUM		64

#define I386		0
#define	I386_4		1
#define	I386_8		2
#define	I486		4
#define	I486_8		5
#define	I486_16		6
#define	I486_32		7
#define PENT		8
#define	PENT_8		9
#define	PENT_16		10
#define	PENT_32		11
#define	PENT_64		12

#define	VIRTUAL		13
#define	JAVA		14

  /*
  ** Operating Systems ************************************
  */

#define	SO_NUM		16

#define	WINDOWS		0
#define	WIN311		1
#define	WIN95		2
#define	WIN_NT		3
#define SYS_LINUX       4   
#define SYS_FREEBSD     5
#define SYS_SOLARIS     6

  /* GNU EDMA Datatypes */

#define DT_EUINT8      0
#define DT_EUINT16     1
#define DT_EUINT32     2
#define DT_ESINT8      3
#define DT_ESINT16     4
#define DT_ESINT32     5
#define DT_EBYTE       6
#define DT_EWORD       7
#define DT_EDWORD      8
#define DT_ECHAR       9
#define DT_EBOOL      10

#define DT_EREAL64    12
#define DT_EZSTRING   13
#define DT_EBUFFER    14
#define DT_EOBJECT    15
#define DT_EUSER      16

  typedef struct {
    HMEM     h;
    EUint32  Size;
    EPVoid   dat;
  } EDMAT_BUFFER;
  
                                                    
#define E		0
#define L		1
#define E_L		2

  /* Operation Identifiers... Virtual Object operations*/
  
#define	WRITE_PROP	0
#define READ_PROP	1
#define RUN_MET		2
#define GET_DATA	3

  typedef ESint32	    SOID;
  typedef ESint32	    MAQID;
  typedef ESint32     CLASSID;
  typedef ESint32     VERSIONID;
  typedef	ESint32	    GROUPID;
  typedef	ESint32	    LEVELID;
  
  typedef ESint32     OBJID;
  typedef EPVoid      POBJ;
  
  typedef HMEM        BSEM;
  typedef void        (*PPROC)();
  
  typedef struct {
    EPVoid	a;	/* Parameter */
    EPfChar	Id;	/* Call identifier */ 
    EUint16	IdPrim;	/* Priomitive Identifier*/
    OBJID	IdObj;	/* Object getting primitive. */
    EUint32	Ind;	/* Property-Method Index. To use level 1 primitives*/
  } VIR_MET_PARAMS;

  /* Not yet implemented.. Maybe removed in future versions*/
  typedef struct t_item {
    HMEM              hBlock;
    ESint32           IdObj;
    HMEM              hBuffer;
    EPVoid            Buffer;
    EUint32           Size;
    EPVoid            *n;
    EPVoid            *p;
  } NOT_ITEM;

  /* Class Information for dynamic class registering*/
  typedef struct {
    EPChar		ClassName;
    EPChar		SOName;
    EPChar		MaqName;
    EPChar		IDFName;
    EPChar		SIUName;
    EByte		IsIDFParser;
    EByte		IsSIUProxy;
    EByte		IsEMIComp;
  } CLASS_INFO;

#ifndef INIMAN_C
  typedef void *PINIFILE;
#endif

  /* GNU EDMA API functions*/
  CLASSID 	EDMAPROC GetClaseName (EUint32,EPChar);
  ESint32 	EDMAPROC GetClaseModule (EUint32,EPChar);
  SOID	EDMAPROC GetClaseSOId (EUint32);
  MAQID	EDMAPROC GetClaseMaqId (EUint32);
  EUint32 	EDMAPROC GetRegClases  (void);
  CLASSID	EDMAPROC GetClassId  (EPChar);
  
  ESint32 	EDMAPROC GetPropNum (EUint32);
  ESint32 	EDMAPROC GetPropName (EUint32,EUint32,EPChar);
  ESint32 	EDMAPROC GetPropType (EUint32,EUint32,EPChar);
  ESint32 	EDMAPROC GetPropId (EUint32 ,EPChar);
  
  ESint32 	EDMAPROC GetMetNum (EUint32);
  ESint32 	EDMAPROC GetMetName (EUint32,EUint32,EPChar);
  ESint32 	EDMAPROC GetMetSig (EUint32,EUint32,EPChar);
  ESint32 	EDMAPROC GetMetIType  (EUint32,EUint32,EPChar);
  ESint32 	EDMAPROC GetMetOType  (EUint32,EUint32,EPChar);
  ESint32 	EDMAPROC IsMetVirtual  (EUint32,EUint32);
  ESint32 	EDMAPROC GetMetId  (EUint32 ,EPChar);
  
  
  /* Shared Memory Primitives */
  HMEM 	EDMAPROC SAlloc  (EUint32,EPChar);
  void 	EDMAPROC SFree  (HMEM,EPVoid);
  EPVoid 	EDMAPROC SGet (HMEM);
  
  /* Private Memory Primitives */
  HMEM 	EDMAPROC PAlloc (EUint32);
  void 	EDMAPROC PFree (HMEM,EPVoid);
  EPVoid 	EDMAPROC PGet (HMEM);
  HMEM     EDMAPROC PRealloc (HMEM,EUint32);

  EUint32  EDMAPROC GetMaqId  (EPChar);
  EUint32	EDMAPROC GetSOId  (EPChar);
  EUint32	EDMAPROC GetMaqName  (EUint32,EPChar*);
  EUint32	EDMAPROC GetSOName  (EUint32,EPChar*);
  EUint32 	EDMAPROC nSO (void);
  EUint32 	EDMAPROC nMaq (void);
  
  OBJID 	EDMAPROC NewObj  (EPChar,EPVoid);
  EUint32 	EDMAPROC FreeObj  (OBJID);
  EDWord 	EDMAPROC MakeVirtual  (OBJID );
  EUint32 	EDMAPROC SetFinal  (OBJID,EUint32);
  
  /* Object information retriving */
  POBJ 	EDMAPROC GetPObj  (OBJID Id);
  OBJID 	EDMAPROC GetObjId  (POBJ p);
  EPChar 	EDMAPROC GetObjClassName (OBJID ,EPChar *);
  
  /* Named Objects. Not included in current release */
  void 	EDMAPROC SetObjName (OBJID,EPChar);
  OBJID 	EDMAPROC GetIdObjbyName  (EPChar);
  EPChar 	EDMAPROC GetObjName (OBJID,EPChar*);
  
  OBJID 	EDMAPROC GetParentId (OBJID Id);
  OBJID 	EDMAPROC GetPseudoParentId  (OBJID);
  /*EPVoid EDMAPROC GetParent (OBJID ); */
  CLASSID 	EDMAPROC GetObjClassId (OBJID Id);
  EByte 	EDMAPROC GetObjStatus (OBJID Id);

  EUint32 	EDMAPROC GetNObj (void);
  EUint32 	EDMAPROC GetObjNumSC (OBJID );
  OBJID 	EDMAPROC GetObjSC (OBJID ,EUint32 );
  OBJID 	EDMAPROC GetObjApp (OBJID );
  EUint32 	EDMAPROC GetAppId  (void);
  EUint32 	EDMAPROC ObjReport (OBJID);
  
  /* Level 1 primitives */
  EPVoid 	EDMAPROC WProp1  (OBJID,EUint32,...);
  EPVoid 	EDMAPROC RProp1  (OBJID,EUint32,...);
  /*EUint32 EDMAPROC Met1 (OBJID,EUint32,EDWord,EByte); */
  EUint32 	EDMAPROC Met1 (OBJID,EUint32,EByte,EPVoid);


  /* Level 2 primitives. Not included in current release. To remove */
  /*
    EPVoid 	EDMAPROC WProp2  (OBJID,EPChar,EDWord);
    EPVoid 	EDMAPROC RProp2  (OBJID,EPChar,EPDWord);
    EUint32 	EDMAPROC Met2  (OBJID,EPChar,EDWord);
  */

  /* Dynamic Inheritance */
  EUint32 	EDMAPROC DeriveClass (EPChar,EPChar*,EPChar *);
  
  EUint32 	EDMAPROC AddSCObj 	(POBJ,EPChar*,EPChar*);
  EUint32 	EDMAPROC OverSCObj  (POBJ,EUint32,EPChar,EPChar);
  EUint32 	EDMAPROC MutateObj  (POBJ,EPChar*,EPChar*);
  
  EUint32 	EDMAPROC MergeObj (OBJID,EPChar,OBJID);
  EUint32 	EDMAPROC FreeSC  (OBJID,EPChar);
  
  EUint32 	EDMAPROC AddSCObj1 	(POBJ,EPChar*,EPChar*);
  EUint32 	EDMAPROC OverSCObj1  (POBJ,EUint32,EPChar,EPChar);
  EUint32 	EDMAPROC MutateObj1  (POBJ,EPChar*,EPChar*);
  
  EUint32 	EDMAPROC MergeObj1 (OBJID,EPChar,OBJID);
  EUint32 	EDMAPROC FreeSC1  (OBJID,EPChar);
  /* Level 3 Primitives */
  
  EPVoid 	EDMAPROC WProp3  (OBJID,EPChar,...);
  
  EPVoid 	EDMAPROC RProp3  (OBJID,EPChar,...);
  EPVoid 	EDMAPROC Met3  (OBJID,EPChar,...);
  EPVoid 	EDMAPROC Met3S  (OBJID,EPChar,EPChar,...);
  /* EPVoid EDMAPROC GetRef3 (POBJ); */
  EPVoid 	EDMAPROC GetRef3  (OBJID);

  /*
    POBJ EDMAPROC GetSubObj (POBJ ,EPChar ,EPChar ,EPChar);
    POBJ EDMAPROC GetSubObj1 (POBJ,EPChar,EPChar,PVIR_DATA *,EByte,EPChar);
    PVIR_DATA EDMAPROC	GetVirtualData (POBJ,EPChar,PVIR_DATA *,EByte *);
  */
  /* Virtual Methods */

  EUint32 	EDMAPROC OverMet  (OBJID,EPChar,POBJ,PPROC);
  EUint32 	EDMAPROC OverMet3  (OBJID,EPChar,EPChar);
  EUint32 	EDMAPROC RestMet  (OBJID,EPChar);
  EUint32 	EDMAPROC OldMet3  (OBJID,EPChar,...);
  
  /* Method override with Virtual Objects */
  EUint32 	EDMAPROC VObjOverMet  (OBJID,EPChar,PPROC);
  EUint32 	EDMAPROC VObjOverMet3  (OBJID,EPChar,EPChar);
  EUint32 	EDMAPROC VObjRestMet  (OBJID,EPChar);
  
  EUint32 	EDMAPROC VObjOldMet3  (OBJID,EPChar,...);
  EUint32 	EDMAPROC VObjOldMet3V  (OBJID,EPChar,EPVoid);
  /* Property override in Virtual Objects */
  EUint32 	EDMAPROC OverProp (OBJID,EPChar,PPROC,EByte);
  EUint32 	EDMAPROC OverProp3 (OBJID,EPChar,EPChar,EByte);
  EUint32 	EDMAPROC RestProp (OBJID,EPChar,EByte);
  
  EUint32 	EDMAPROC OldWProp3 (OBJID IdObj,EPChar Id,...);
  EUint32 	EDMAPROC OldWProp3V (OBJID IdObj,EPChar Id,EPVoid);
  EUint32 	EDMAPROC OldRProp3 (OBJID,EPChar,...);
  
  /*
    // Hebras portables
    HMEM 	EDMAPROC EDMAThread)(EPVoid,EPVoid,EPUint32);
    void 	EDMAPROC EDMAExitThread)(EUint32 );
    EUint32 	EDMAPROC EDMAGetThreadSelfId)(void);
    HMEM 	EDMAPROC EDMAGetThreadSelf)(void);
    EByte 	EDMAPROC EDMASetThreadPriority)(HMEM ,EUint32 );
    EUint32 	EDMAPROC EDMAGetThreadPriority)(HMEM );
    
    // Metodos enhebrador
    EPVoid   EDMAPROC EMet3) (OBJID ,EPChar,EDWord);
    
    // Notificaciones Asncronas
    EUint32 	EDMAPROC SetObjNotifyFunc)(OBJID ,EPVoid);
    EUint32 	EDMAPROC DeactivateNotify) (OBJID );
    EUint32 	EDMAPROC NotStub)(EPVoid );
    EUint32 	EDMAPROC SendNotify) (OBJID,OBJID,EPChar,EPVoid,EUint32);
    
    // Semaforos binarios
    BSEM 	EDMAPROC CreateBSem)(EPChar Name,EUint32,EUint32);
    void 	EDMAPROC FreeBSem)(BSEM bSem);
    void 	EDMAPROC WaitBSem) (BSEM bSem);
    void 	EDMAPROC SignalBSem) (BSEM bSem);
  */

  /* Multi IDF Interface. IDF Parsers */
  CLASSID 	EDMAPROC IDFGetClassID ();
  ESint32 	EDMAPROC IDFSetGeneral (CLASSID ,EPChar,EPChar,EPChar,EPChar);
  ESint32 	EDMAPROC IDFSetDef (CLASSID,EUint32,EUint32,EUint32);
  ESint32 	EDMAPROC IDFSetProp  (CLASSID,EUint32,EPChar,EPChar,EPChar,EUint32);
  ESint32 	EDMAPROC IDFSetMet  (CLASSID iC,EUint32 iP,EPChar ,EPChar,EByte,EByte,EByte);
  ESint32 	EDMAPROC IDFSetSCList  (CLASSID ,EPChar	*);
  ESint32 	EDMAPROC IDFSetClassID  (CLASSID );

  /* Stock Class management */
  EUint32 	EDMAPROC AddStockClass  (CLASS_INFO ,EPChar,EPChar);
  EUint32 	EDMAPROC DelStockClass  (EPChar);

  /* EMI Extensions */
  ESint32 	EDMAPROC GetEMIComp  (EPChar);
  EUint32 	EDMAPROC AddEMIComp  (EPChar,EPChar);
  EUint32 	EDMAPROC HookGetClass (CLASSID);

  /* Miscelanea */
  EUint32 	EDMAPROC ECout  (EPChar);
  EUint32 	EDMAPROC SetAppName (EPChar);
  EPChar 	EDMAPROC GetSystemPath (EPChar );
  
  EUint32 	EDMAPROC SetDebugLevel  (EUint32 l);
  EUint32 	EDMAPROC ECoutObj (OBJID IdObj,EPChar f,...);
  EUint32 	EDMAPROC ECout1 (EPChar f,...);
  EUint32 	EDMAPROC ECoutErr (EPChar f,...);
  EUint32 	EDMAPROC ECoutDbg (EUint32 l,OBJID IdObj,EPChar f,...);
  
  /* EDMA Buffer type management */
  ESint32  EDMAPROC EBufferAlloc (EDMAT_BUFFER*,EUint32);
  ESint32  EDMAPROC EBufferFree (EDMAT_BUFFER*);
  ESint32  EDMAPROC EBufferRealloc (EDMAT_BUFFER *,EUint32);  

  /* Level 2 Extended Primitives */
  EPVoid 	EDMAPROC EDMAWProp3  (OBJID,EPChar,EPVoid);
  EPVoid 	EDMAPROC EDMARProp3  (OBJID,EPChar,EPVoid);
  EPVoid 	EDMAPROC EDMAMet3  (OBJID,EPChar,EPVoid);
  EPVoid 	EDMAPROC EDMAMet3S (OBJID,EPChar,EPChar,EPVoid);
  
  /* Aditional Properties query functions */
  ESint32 EDMAPROC GetPropTypeId (CLASSID,EUint32);
  ESint32 EDMAPROC GetPropTypeSig (CLASSID,EUint32,EPChar);
  ESint32 EDMAPROC GetPropNElems (CLASSID,EUint32);
  ESint32 EDMAPROC GetTypeSize (EUint32);
  EUint32 EDMAPROC GetTypeSig (EUint32,EPChar);
  ESint32 EDMAPROC GetPropId3 (OBJID,EPChar,CLASSID*);

  /* Class Interface Management functions*/
  ESint32 EDMAPROC LoadClassInt (CLASSID);
  ESint32 EDMAPROC UnloadClassInt (CLASSID);   

  /* Ini file management functions*/
  PINIFILE EDMAPROC OpenINI (EPChar);
  EUint32  EDMAPROC GetINIString (PINIFILE,EPChar,EPChar,EPChar,EPChar,EUint32);
  EUint32  EDMAPROC GetINIInt (PINIFILE,EPChar,EPChar,EUint32);
  EUint32  EDMAPROC CloseINI (PINIFILE);

  /* Static Methods*/
  EPVoid EDMAPROC SMet3 (EPChar,EPChar,...);
  EPVoid EDMAPROC SMet3x (CLASSID,EPChar,EPVoid);

  EPVoid EDMAPROC SMet3S (EPChar,EPChar,EPChar,...);
  EPVoid EDMAPROC SMet3Sx (CLASSID, EPChar,EPChar,EPVoid);

  /* Casting methods */
  OBJID  EDMAPROC UpCastObj(OBJID, EPChar);
  OBJID  EDMAPROC DownCastObj(OBJID, EPChar);
  OBJID  EDMAPROC CastObj(OBJID, EPChar);

  /* Clonning, not yet implemented.... we're in the way*/
  OBJID  EDMAPROC CloneObj(OBJID);
  OBJID  EDMAPROC ShallowCloneObj(OBJID);
  
  ESint32 EDMAPROC GetNParsers (void);
  CLASSID EDMAPROC GetParserClass (ESint32 i);
  
  ESint32 EDMAPROC IsIDFParser (CLASSID id);
  ESint32 EDMAPROC IsSIUProxy (CLASSID id);
  ESint32 EDMAPROC IsEMIHandler (CLASSID id);

  ESint32 EDMAPROC GetNProxies (void);
  CLASSID EDMAPROC GetProxyClass (ESint32 i);
  ESint32 EDMAPROC GetNEMIHandlers (void);
  CLASSID EDMAPROC GetEMIHandlerClass (ESint32 i);

  EUint32 EDMAPROC EDMAEnd(void);   
  EUint32 EDMAPROC EDMAInit(void);   
  
  HMEM EDMAPROC EDMARTLink(void);
  
#ifdef __cplusplus
}
#endif

#endif
