/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 Entorno de Desarrollo Modular y Abierto
 (c) David Martnez Oliveira
 Versin Beta 0.3r1
 15 de Junio de 1997

 Gestin de Clases

 REVISIONES:-------------------------------------------------------------
 15 de Junio de 1997
 -----------------------
 4 de Agosto de 1997
 Aadimos soporte para signaturas.
 El nombre de la funcin nativa en la DLL ser la concatenacin
 del identificador de la funcin y de la signatura.
 -----------------------------------------
 8 de AGosto de 1997
 Modificacin para mapeo de cdigo por proceso
 Lectura de los punteros a las funciones e instancia de DLL en pClase
 -----------------------------
 6 de Octubre de 1997
 Aadimos funcin para registrar clases de STOCK
---------------------------------------------------------
3 de Enero de 1999
Aadimos funciones para carga y descarga de interfaces de clases
Por ahora no vamos a considerar el caso de la herencia esttica. SUpuestamente este hecho se gestionar de forma autintica desde las llamadas a FreeObj
-------------------------------------------------------------------
10 de Mayo de 1999
Modificacin para control de estado de clases por proceso.
------------------------------------------------------------------
19 de Febrero de 2000
Se elimina la carga de interface durante el registro de clases. Ahora la
carga se realiza bajo demanda
 * -----------------------------------------------
 * January, 31th, 2001
 * Code cleanup and comment translation
 * -------------------------------------------------------
 * July, 4th, 2001
 * First modifications to allow interface definitions. We're trying
 * to allow class loading for classes which have pure virtual methods,
 * that is, class with no definition for some methods
 *
 * Fixed problem loading class implementation for pure abstract class
 * which haven't implementation associated.
 *
 * This code still has not been tested... I must to sleep :P
 *
 * FIXME: AddStockClass. We need to update the CLASSINFO struct used
 * for this functions.
 * ------------------------------------------------------------------
 * October, 19th,2001
 * Removing warnnings
 * Changes AddStock class. Now don't prepend system path. This was
 * causing ines_class_register to introduce wrong information in the
 * registry
 */

#include <stdio.h>
#include "portable.h"
#include "vglobal.h"
#include "idf.h"
#include "classq.h"
#include "const.h"
#include "multiidf.h"
#include "dynlink.h"
#include "clas.h"

/* Local Prototypes */
ESint32 EDMAPROC void_func (OBJID id);

ESint32 EDMAPROC LoadClassImp (CLASSID IdC)
{
  EUint32	i,n;
  HMEM	        h;
  EChar		cadena[80],fname[80];
  PPROC		func;
  EByte		flag,HasOnLoad;
  EUint32	(EDMAPROC *EDMARTLink1)(PPROC *);
  
  /* Checks if the class is already loaded for this process
     This is private information for each process */
  if (ProcMapTable[IdC]==CLASS_LOADED)
    {
      ECoutDbg (8,-1,"[LoadClassImp] Implementation for Class %d already in memory",IdC);
      return 0;
    }
  h=(HMEM)NULL;    /* Initialize shared library handler for PureAbstract classes*/
  if (gClase[IdC].PureAbstract==0) /* If the class isn't a Pure Abstract class nor an Interface*/
    {
      /* First of all, we load the code module */
      /*ECout1 ("[DEBUG] Loading shared library : %s",gClase[IdC].SysClass.ModuleName);*/
      h=LoadLib(gClase[IdC].SysClass.ModuleName);   
      if (h==(HMEM)NULL)
	{
	  ECoutErr ("[LoadClassImpl] Can't load module %s for class %s",gClase[IdC].SysClass.ModuleName,gClase[IdC].ClassName);
	  return -1;
	}
      
      /*sprintf (cadena,"\nApp(%d) loads %s implementation with handle %p",AppId,gClase[IdC].ClassName,h);*/
      
      /* We obtain the entry point for the class */
      /*  Temporally remove this... it seems it isn't necessary
      EDMARTLink1=GetAddress(h,"_EDMARTLink1");
      if (EDMARTLink1==NULL)
	ECoutErr ("[LoadClassImp] Cant set EDMA entry points");
      EDMARTLink1(ProcEPTable);
      */
    }
  pClase[IdC].hLib=h;
  /* Checks if the intertace is already loaded and mapped for this process */
  if (ProcMapTable[IdC]<CLASS_IMAPPED)
    {
      ECoutDbg (8,-1,"[LoadClassImp] Interface for class %d isn't loaded (%d)....",IdC,gClase[IdC].Status);
      /* if it isn't, we hace to load it */
      LoadClassInt(IdC);
    }
  /* We read the function pointers for class methods */
  n=gClase[IdC].nMet;
  flag=0;
  HasOnLoad=0;
  for (i=0;i<n;i++)
    {
      fname[0]=0;
#ifdef WINAPI_32	   
      strcpy (fname,"_");
#endif	   
      /* Test for OnLoad static method while mapping class*/
      if (!HasOnLoad)
	if (strcmp(pClase[IdC].Met[i].IdMet,"OnLoad")==0)
	  HasOnLoad=1;
      strcat (fname,gClase[IdC].ClassName);
      strcat (fname,pClase[IdC].Met[i].IdMet);
      /* We add the method's signature */
      strcat (fname,pClase[IdC].Met[i].Sign);
      if (pClase[IdC].Met[i].Abstract) {
	func=(PPROC)void_func;
	/*ECout1 ("[LoadClassImpl] Assigning void_func to method %s",fname);*/
      }
      else
	func=GetAddress(h,fname);   

      if (func==NULL)
	{
	  /* If we can locate implementation, we assign void_func*/
	  flag=1;
	  ECoutErr ("[LoadClassImpl] Cant locate method %s for class %s in module %s, and method isn't virtual",fname,gClase[IdC].ClassName,gClase[IdC].SysClass.ModuleName);
	}
      fClase[IdC].Met[i].Func=(PPROC*)func;
      
    }
  
  /* The loaded status only makes sense for each process, 
     so we only update ProcMapTable */
  ProcMapTable[IdC]=CLASS_LOADED;
  ECoutDbg(8,-1,"[LoadClassImpl] (%s)Class Implementation  succesfully mapped",gClase[IdC].ClassName);
  /* Now, we look for a CLASEOnload function*/
  if (HasOnLoad) {
    SMet3(gClase[IdC].ClassName,"OnLoad");
  }
  return flag;
}

ESint32 EDMAPROC FreeClassImp (CLASSID IdC)
{
  EChar	        cadena[80],fname[80];
  EByte         HasUnload;
  ESint32       i,n;
  
  if (ProcMapTable[IdC]==CLASS_LOADED)
    {
      /* Look for available OnUnload Method*/
      n=gClase[IdC].nMet;
      for (i=0;i<n;i++)
	if (strcmp(pClase[IdC].Met[i].IdMet,"OnUnload")==0)
	  break;
      if (i!=n)
	SMet3(gClase[IdC].ClassName,"OnUnload");
      if (gClase[IdC].PureAbstract==0) /* We only must to unload if we've something to unload*/
	UnloadLib(pClase[IdC].hLib);   
      /* Actually we only change the interface's status */
      ProcMapTable[IdC]=CLASS_IMAPPED;
      /* The Interface is still defined and mapped for the process after
	 the class code is unloaded from memory */
    }
  
  return 0;
  
}

EUint32 EDMAPROC AddStockClass (CLASS_INFO ci,EPChar IDFName,EPChar IMPName)
{
  CLASSID		cId;
  OBJID		idParser;
  EUint32		Id;

  /* Checks if class id is avaliable */
  cId=GetClassId(ci.ClassName);
  if (cId!=-1) 
    {
      ECoutErr ("[AddStockClass] This ClassName already exists");
      return -1;
    }
  
  cId=IDFGetClassID();	/* Gets a class ID */
  if (cId==-1)
    {
      ECoutErr("[AddStockClass] Cant get class identifier");
      return -1;
    }

  /* Inits general data for the class */
  strcpy(gClase[cId].ClassName,ci.ClassName);
  gClase[cId].MaqId=GetMaqId(ci.MaqName);
  gClase[cId].SOId=GetSOId(ci.SOName);
  gClase[cId].SOId=GetSOId(ci.SOName);
  /*
  strcpy (gClase[cId].SysClass.ModuleName,GVar->SystemPath);
  strcat (gClase[cId].SysClass.ModuleName,IMPName);
  */
  strcpy (gClase[cId].SysClass.ModuleName,IMPName);

  /* Sets default parser (EDMAIDF) for this class */
  gClase[cId].IDFParser=0;		
  if (strcmp(ci.IDFName,"EDMAIDF")!=0)
    {
      gClase[cId].IDFParser=GetParser(ci.IDFName);
    }
  /* By default, the class is native... so no SIU proxy is defined*/
  gClase[cId].SIUProxy=0;		
  if (ci.SIUName!=0)
    if (strlen(ci.SIUName)!=0)
      {
	gClase[cId].SIUProxy=GetProxy(ci.SIUName);
      }

  gClase[cId].IsIDF=ci.IsIDFParser;
  gClase[cId].IsSIU=ci.IsSIUProxy;
  gClase[cId].IsEMI=ci.IsEMIComp;
  
  /* At this point the class is totally defined */
  GVar->nClases++;
  ECoutDbg (8,-1,"[AddStockClass] Class %s added...",ci.ClassName);
  ECoutDbg (8,-1,"[AddStockClass] Ready for getting interface");

  /* Mark the class for adding to the EDMA registry on shutdown*/
  gClase[cId].IsStock=1;
  gClase[cId].Status=CLASS_DEF;
  ECoutDbg(8,-1,"[AddStockClass] Class added with id %d",cId);

  return cId;
}

EUint32 EDMAPROC DelStockClass (EPChar Name)
{
  CLASSID		IdC;
  
  IdC=GetClassId(Name);
  /* We only can delete a class if it isn been used */
  if (gClase[IdC].Ocurrences==0)		
    {
      /* We mark it like free and we free associated memory */
      gClase[IdC].Status=CLASS_FREE;
      SFree(pClase[IdC].SysClass.hSCList,pClase[IdC].SCList);
      SFree(pClase[IdC].SysClass.hProp,pClase[IdC].Prop);
      SFree(pClase[IdC].SysClass.hMet,pClase[IdC].Met);
      SFree(pClase[IdC].SysClass.hNot,pClase[IdC].Not);	   
    }
  GVar->nClases--;
  if (GVar->nClases<0)
    {
      ECoutErr("[DelStockClass] Number of clases less than zero !!!");
      
    }
  return 0;
}

/*
 * New Functions for Class Interface load and unload
 */

ESint32 EDMAPROC LoadClassInt (CLASSID i)
{
  OBJID  idParser;

  ECoutDbg(8,-1,"[LoadClassInt] Loading interface for class %d",i);
  /* If the interface is already loaded we do nothing */
  if (ProcMapTable[i]>=CLASS_ILOADED)
    {
      ECoutDbg(8,-1,"[LoadClassInt] Class interface %d already in memory",i);
      return 0;
    }
  if (gClase[i].IDFParser==0)
    {
      GetInterface(i,gClase[i].ClassName);	     
    }
  
  else
    {
      ECoutDbg (40,-1,"      [LoadClassInt] Invoking IDF Parser %s",GVar->IDFParser[gClase[i].IDFParser].Id);
      idParser=NewObj(GVar->IDFParser[gClase[i].IDFParser].Id,NULL);
      ECoutDbg (40,-1,"      [LoadClassInt] Parser created");
      Met3(idParser,"Parse",i);
      ECoutDbg(40,-1,"       [LoadClassInt] Parssing done");
      FreeObj(idParser);
      
    }
  ECoutDbg(8,-1,"[LoadClassInt] Interface succesfully loaded");
  /* The class's interface is created in shared memory so it can be
     acceded by all the process*/
  gClase[i].Status=CLASS_ILOADED;
  /* For the current process executing the function, it is mapped too*/
  ProcMapTable[i]=CLASS_IMAPPED;
  return 0;
}
ESint32 EDMAPROC UnloadClassInt (CLASSID IdC)
{
  
  ECoutDbg(8,-1,"[LoadClassInt] Unloading interface for class %d",IdC);
  FreeInterface (IdC,gClase[IdC].ClassName);
  ProcMapTable[IdC]=CLASS_DEF;
  return 0;
}
/*
 * void_func for interface definition
 */

ESint32 EDMAPROC void_func (OBJID id) 
{
  ECoutObj (id,"[SYSTEM_WAN] void_func: your abstract class isn't complete");
  return -1;
}
