/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 Entorno de Desarrollo Modular y Abierto
 Versin Beta 0.3r1
 (c) David Martnez Oliveira

 Modulo de Query de Clases
 Revisiones: ---------------------------------------------------------
 15 de Mayo de 1997
 * -------------------
 * 27/11/1997
 *   Modificaciones menores. Tipos de parmetros consistentes con lo
 * que representan, y gestin de errores
 *
 * -------------------------------
 * 19/12/1997
 * 
 *   Aadimos ms funciones de query, necesarias para FAYE
 * ----------------
 * 8/1/1998
 *   Ms funciones de query requeridas por FAYE. GetPropId3
 * -----------------------
 * 12/1/1998
 *   Vamos a modificar la primitiva GetPropId3 para que nos devuelva
 * el identificador de la clase y el indice de la propiedad especificado
 * en un camino de clase.
 * 
 *   La nueva implementacin no utilizar el objeto para localizar los
 * datos, simplemente trocear la cadena y se quedar con los datos del
 * final. Ser la invocacin de WProp3 o RProp3 la que determine que el
 * camino es incorrecto.
 * 
 *   Hacemos esto as para la implementacin de WProp3 en FAYEPROXY ya
 * que en el sistema local tenemos solo una representacin parcial de la
 * tabla de superclases de los objetos.
 * ---------------------------------------------------
 * January, 31th, 2001
 * Some cosmetic changes in the code, erasing non used code
 * ----------------------------------------------------------
 * November, 11th,2001
 * Added IsMetAbstract and IsMetStatic primitives
 * ------------------------------------------------------------
 * November, 24th, 2001
 * Adding IsISFParser, IsSIUProxy, IsEMIHandler primitives
*/

#include <string.h>
#include "portable.h"
#include "vglobal.h"
#include "tclass.h"

ESint32 EDMAPROC GetClaseName (CLASSID Id,EPChar nombre)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  strcpy (nombre,gClase[Id].ClassName);
  return 0;
}

SOID EDMAPROC GetClaseSOId (CLASSID Id)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  return gClase[Id].SOId;
}

MAQID EDMAPROC GetClaseMaqId (CLASSID Id)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  return gClase[Id].MaqId;
}
ESint32 EDMAPROC GetClaseModule (CLASSID Id,EPChar nombre)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  strcpy (nombre,gClase[Id].SysClass.ModuleName);
  return 0;
}
EUint32 EDMAPROC GetRegClases (void)
{
  return GVar->nClases;
}

CLASSID EDMAPROC GetClassId (EPChar Name)
{
  EUint32		i,r,j;
  OBJID		idEMI;
  
  j=0;
  for (i=0;j<GVar->nClases;i++)
    {
      if (gClase[i].Status!=CLASS_FREE)
	{
	  if (strcmp(gClase[i].ClassName,Name)==0)
	    break;
	  j++;
	}
    }
  if (i==GVar->nClases)
    {
      r=-1;
      if (GVar->GetClassEMI!=0)
	{
	  idEMI=NewObj(gClase[GVar->GetClassEMI].ClassName,NULL);
	  r=Met3(idEMI,"GetClass",Name);
	  FreeObj(idEMI);
	}
      return r;
    }
  else
    return i;
}

ESint32 EDMAPROC  IsIDFParser (CLASSID id) {
  if ((id<0) || (id>GVar->nClases))
    return -1;
  return gClase[id].IsIDF;
}

ESint32 EDMAPROC IsSIUProxy (CLASSID id) {
  if ((id<0) || (id>GVar->nClases))
    return -1;
  return gClase[id].IsSIU;  
}

ESint32 EDMAPROC IsEMIHandler (CLASSID id) {
  if ((id<0) || (id>GVar->nClases))
    return -1;
  return gClase[id].IsEMI;
}

ESint32 EDMAPROC	GetPropNum(CLASSID Id)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  return gClase[Id].nProp;
}

ESint32 EDMAPROC	GetPropName(CLASSID Id,EUint32 Ind,EPChar Name)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  if (Ind>gClase[Id].nProp)
    return -1;
  strcpy (Name,pClase[Id].Prop[Ind].IdProp);
  return 0;
}
ESint32 EDMAPROC GetPropId (CLASSID Id,EPChar Name)
{
  EUint32		i;
  if ((Id<0) || (Id>GVar->nClases))
    return -1;

  for (i=0;i<gClase[Id].nProp;i++)
    if (strcmp(pClase[Id].Prop[i].IdProp,Name)==0)
      break;
  if (i==gClase[Id].nProp)
    return -1;
  else
    return i;
}

ESint32 EDMAPROC	GetPropType(CLASSID Id,EUint32 Ind,EPChar Name)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  strcpy (Name,tipo[pClase[Id].Prop[Ind].Tipo].Id);
  return 0;
}
ESint32 EDMAPROC	GetMetNum(CLASSID Id)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  return gClase[Id].nMet;
}

ESint32 EDMAPROC	GetMetName(CLASSID Id,EUint32 Ind,EPChar Name)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  strcpy (Name,pClase[Id].Met[Ind].IdMet);
  return 0;
}
ESint32 EDMAPROC	GetMetSig(CLASSID Id,EUint32 Ind,EPChar Name)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  strcpy (Name,pClase[Id].Met[Ind].Sign);
  return 0;
}

ESint32 EDMAPROC	GetMetIType(CLASSID Id,EUint32 Ind,EPChar Name)
{
  return 0;
}

ESint32 EDMAPROC	GetMetOType(EUint32 Id,EUint32 Ind,EPChar Name)
{
  return 0;
}

ESint32 EDMAPROC IsMetVirtual(CLASSID Id,EUint32 Ind)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  if (pClase[Id].Met[Ind].Virtual)
    return 1;
  else
    return 0;
}

ESint32 EDMAPROC IsMetAbstract(CLASSID Id,EUint32 Ind)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  if (pClase[Id].Met[Ind].Abstract)
    return 1;
  else
    return 0;
}

ESint32 EDMAPROC IsMetStatic(CLASSID Id,EUint32 Ind)
{
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  if (pClase[Id].Met[Ind].Static)
    return 1;
  else
    return 0;
}

ESint32 EDMAPROC GetMetId (CLASSID Id,EPChar Name)
{
  EUint32		i;
  
  if ((Id<0) || (Id>GVar->nClases))
    return -1;
  for (i=0;i<gClase[Id].nMet;i++)
    if (strcmp(pClase[Id].Met[i].IdMet,Name)==0)
      break;
  if (i==gClase[Id].nMet)
    return -1;
  else
    return i;
}

/*
 * New Class Query Functions
 * 10/12/1997
 * 
 */

ESint32 EDMAPROC GetPropTypeId (CLASSID Id,EUint32 IdProp)
{
  return pClase[Id].Prop[IdProp].Tipo;
}

ESint32 EDMAPROC GetPropTypeSig (CLASSID Id,EUint32 IdProp,EPChar Sig)
{
  strcpy (Sig,tipo[pClase[Id].Prop[IdProp].Tipo].Sig);
  return 0;
}

ESint32 EDMAPROC GetPropNElems (CLASSID Id,EUint32 IdProp)
{
  return pClase[Id].Prop[IdProp].nElem;
}

EUint32 EDMAPROC GetTypeSize(EUint32 tip)
{
  return tipo[tip].tam;
}

EUint32 EDMAPROC GetTypeSig(EUint32 tip,EPChar Name)
{
  strcpy (Name,tipo[tip].Sig);
}

/*
 * Old GetPropId3 function, comment for if the flies
 */
/*
ESint32 EDMAPROC GetPropId3 (OBJID IdObj,EPChar Name,CLASSID *IdC)
{
   EUint32		i;
   POBJ                 pObj;
   EChar                LocalId[80];
   CLASSID              Id;
   VIR_DATA             *VirData;
   
   if ((IdObj<0) || (IdObj>GVar->nObj))
     return -1;
//   if (ProcMapTable[Id]=CLASS_FREE)
//     return -1;
   ECout1("(GetPropId3) Executing ...");
   pObj=&gObj[IdObj];
   pObj=GetSubObj1 (pObj,Name,LocalId,&VirData,V_PROP,"");
   Id=pObj->IdClase;
   *IdC=Id;
   for (i=0;i<gClase[Id].nProp;i++)
     if (strcmp(pClase[Id].Prop[i].IdProp,LocalId)==0)
       break;
if (i==gClase[Id].nProp)
	return -1;
else
	return i;
}
 */

// New Implementation for GetPropId3

ESint32 EDMAPROC GetPropId3 (OBJID IdObj,EPChar Name,CLASSID *IdC)
{
  EUint32		i,j;
  POBJ                 pObj;
  EChar                LocalId[80];
  CLASSID              Id;
  VIR_DATA             *VirData;
  EUint32              n;
  EChar                CN[32];  /* Found class name */
  EChar                PN[32];  /* Class which owns the property */
  EPChar               aux1,aux2;
  
  if ((IdObj<0) || (IdObj>GVar->nObj))
    return -1;
  /* We calculate the elements in the class way */
  strcpy (LocalId,Name);
  n=0;
  for (i=0;i<strlen(Name);i++)
    if (Name[i]=='/')
      n++;
  if (n==0)  /* if there is no way, we take class ID from object */
    Id=gObj[IdObj].IdClase;
  else   /* else we get the las class ID */
    {
      /* We go to the "penultimo" identifier */
      aux1=LocalId;
      aux2=strchr(aux1,'/');
      for (i=0;i<n-1;i++)
	{
	  aux1=aux2+1;
	  aux2=strchr(aux1,'/');
	}
      strcpy (PN,aux2+1);
      *aux2=0;
      strcpy (CN,aux1);
      Id=GetClassId(CN);
      strcpy(LocalId,PN);
      
    }
  *IdC=Id;
  for (i=0;i<gClase[Id].nProp;i++)
    if (strcmp(pClase[Id].Prop[i].IdProp,LocalId)==0)
      break;
  if (i==gClase[Id].nProp)
    return -1;
  else
    return i;
}
