/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
 Entorno de Desarrollo Modular y Abierto
 Versin Beta 0.3r1
 (c) David Martnez Oliveira


 Modulo de Gestin de ficheros de interface
 Revisiones: ---------------------------------------------------------
 28 de Mayo de 1997
 ---------------------
 24 de Julio de 1997
 Aadimos soporte para matrices. En el IDF se aade el campo ELEM que indica
 el nmero de elementos del tipo indicado. Si es 0, el tipo es simple, sino es una
 matriz.

 Los errores se envan a la consola con ECout. Habr que modificarlos
 para utilizar una funcin especfica para mostrar errores del sistema.
 --------------------------------
 4 de Agosto de 1997
 Modificamos LeeMetodo para que lea la signatura asociada al mtodo
 ---------------------------------
 8 de Agosto de 1997
 Modificacion para mapeo de cdigo por proceso. Creacin de la tabla pClase
 ----------------------------------------
 6 de Octubre de 1997
 Modificacin para poder leer definiciones de interface de distintos
 directorios.
 * -----------------------
 * 18 de Noviembre de 1997
 *   Modificacin para el soporte de tipos definidos por el usuario.
 * Un tipo definido por el usuario es un clase y debe ser declarada con
 * anterioridad en EDMA32.CFG.
 * 
 *   Almacenamos en el campo UserData de la estructura PROP el identificador
 * de la clase a la que referencia el tipo
----------------------------------------------
14 de Mayo de 1999
  Correctin de las evoluciones de estados de las clases.
  * ------------------------------------------
  * January, 5th, 2001
  * We continue cleaning up the sources
  * ------------------------------------------------------------------
  * April, 15th, 2001
  * Added support for reading static method metainformation
  * -------------------------------------------------------------------
  * July, 4th, 2001
  * LeeMetodo modified to read Abstract field in order to not try to resolve
  * function names for abstract methods in shared libraries
  *
  * Added suport to read information about Pure Abstract/Interface classes
  * This code still has not been tested
  * -------------------------------------------------------------------
  * July, 6th, 2001
  * Instead of adding a new field in the IDF file to mark pure abstract
  * classes, we simply, don't provide implementation information.
  * 
  * This processing is now done in ini1.c
  * -----------------------------------------------------------------
  * November, 10th, 2001
  * Changes default idf file to SystemPath/share/idf
  * TODO: Generalize this to work with namespaces
  * ---------------------------------------------------------------------
  * December, 3rd, 2001
  * GetInterface now gets preferer anchor points from SuperClass lists in
  * IDF's
*/

#include <stdio.h>
#include "portable.h"
#include "idf.h"
#include "iniman.h"
#include "vglobal.h"

EUint32 EDMAPROC GetInterface (EUint32 ClassId,EPChar Name)
{
  CLASSID	CId;
  EChar		FilePath[256];
  static EChar		Buffer[200];
  EChar		Aux[100];
  EChar		Error[80];
  EDWord	Result;
  FILE		*f;
  EUint32	nProp,nMet,nNot,i,j,ii;
  HMEM		h,h1;
  EUint32	DataSize;
  EPChar	*clist;
  EPChar        *idlist,aux;
  PINIFILE      pi;   
  
  /* Get Class identifier*/
  if (ClassId==-1)
    CId=GetClassId (Name);
  else
    CId=ClassId;
  if (CId==-1) {
    ECoutErr ("[GetInterface] Class %s doesn't exist");
    /* FIXME: We must to recover from this errors*/
    return -1;
  }
  /* If interface is already loadeded or implementation is mapped in memory */
  if (ProcMapTable[CId]>=CLASS_ILOADED) {
    ECout1("[GetInterface] Interface for Class %d is already loaded",CId);
    /* We check out implementation mapping for this process*/
    if (ProcMapTable[CId]>=CLASS_IMAPPED)
      return 0;   /* if it's mapped there's nothing to do */
  }
  /*
   * Si el interface de la clase no est cargado
   * Hay que hacerlo todo. Crear y mapear estructuras y almacenar informacin
   * de interface en la memoria compartida.
   *-------------------------------------------------------------------------
   * Localizamos el fichero de definicin de interface
   * si tenemos un path completo lo utilizamos, sino buscamos en el
   * directorio idf de EDMA
   */
  if (strchr(Name,':')==0) {
    strcpy (FilePath,GVar->SystemPath);
#ifdef WINAPI_32	   
    /* FIXME: Probably this location'll change when we rebuild the
       system to windows OSes. We must try to use it with cygwin and maybe
       we can use the same directory tree on windows systemas*/
    strcat (FilePath,"idf\\");
#endif
#ifdef LINUX
    /*strcat (FilePath,"share/edma/idf/");*/
    strcat (FilePath,IDFDIR);
#endif	   
    /* TODO: When supporting namespaces we must to add it here*/
    strcat (FilePath,Name);
    strcat (FilePath,".idf");
  } else
    strcpy(FilePath,Name);

  /* We open the file */
  f=fopen(FilePath,"rt");
  if (f==NULL) {
    ECoutErr ("[GetInterface] Interface definition file %s not found",FilePath);
    return -1;
  }
  fclose (f);
  
  /* Get General Information*/
  pi=OpenINI(FilePath);
  Result=GetINIString (pi,"General","Name",NULL,Buffer,200);
  strcpy (gClase[CId].ClassName,Buffer);
  
  /* Get PureAbstract/Interface info */
  /*
    gClase[CId].PureAbstract=GetINIInt (pi,"Definition","PureAbstract",0);
  */

  /* Get Definitions */
  nProp=GetINIInt (pi,"Definition","PropertiesNum",0);
  nMet=GetINIInt (pi,"Definition","MethodsNum",0);
  nNot=GetINIInt (pi,"Definition","NotificationsNum",0);
  /* We build properties tables */
  strcpy (Aux,gClase[CId].ClassName);
  strcat (Aux,"Prop");
  if (nProp!=0) {
    h=SAlloc(nProp*sizeof(PROP),Aux);
    if (h==(HMEM)0) {
      ECoutErr ("[GetInterface] Can't allocate propieties table");
      return -1;
    }
    
    pClase[CId].Prop=(PROP *)SGet(h); 
    pClase[CId].SysClass.hProp=h;
  }
  
  strcpy (Aux,gClase[CId].ClassName);
  strcat (Aux,"Met");
  if (nMet!=0) {
    h=SAlloc(nMet*sizeof(MET),Aux);
    if (h==(HMEM)0) {
      ECout("[GetInterface] Can't allocate method table");
    }
    
    pClase[CId].Met=(MET*)SGet(h);
    pClase[CId].SysClass.hMet=h;
    
    /* Per process code mapping */
    fClase[CId].hFMet=PAlloc(sizeof(PROCMET)*nMet);
    if (fClase[CId].hFMet==0) {
      ECout ("Cant allocate process method table");
    }
    fClase[CId].Met=(PROCMET*)PGet(fClase[CId].hFMet);
    
  }
  /*
   * Notifications aren't used in current implementations
   * We mantain the code for future implementation
   */
  strcpy (Aux,gClase[CId].ClassName);
  strcat (Aux,"Not");
  if (nNot!=0) {
    h=SAlloc(nNot*sizeof(NOT),Aux);
    if (h==(HMEM)0) {
      ECoutErr ("[GetInterface] Can't allocate notification table");
    }
    
    pClase[CId].Not=(NOT*)SGet(h);
    pClase[CId].SysClass.hNot=h;
  }

  DataSize=0;
  for (i=0;i<nProp;i++) {
    pClase[CId].Prop[i].Off=DataSize;
    DataSize+=LeePropiedad(pi,CId,i);
  }
  gClase[CId].TamDatos=DataSize;
  gClase[CId].nProp=nProp;
  
  j=0;
  for (i=0;i<nMet;i++)
    j+=LeeMetodo(pi,CId,i);
  gClase[CId].nMet=nMet;
  gClase[CId].nMetVir=j;
  
  /* We process static inheritance information in interface definition file */
  Buffer[0]=0;   
  Result=0;
  
  Result=GetINIString (pi,"Definition","SCList",NULL,Buffer,200);
  if (Result!=0)  { /* If exists some information */
    /* We count commas to know how many classes to process */
    j=0;
    for (i=0;i<Result;i++)
      if (Buffer[i]==',') j++;
    j++;
    ECoutDbg(12,-1,"[GetInterface] %d superclases located :",j);
    gClase[CId].Derived=j;
    /* Alloc memory for SuperClass List*/
    h=PAlloc(sizeof(EPChar)*(j+1));
    if (h==0) {
      ECoutErr ("[GetInterface] Can't create inheritance info");
      return -1;
    }
    clist=(EPChar*)PGet(h);
    clist[0]=Buffer;
    /* Allocate memory for SuperClass prefered anchor points*/
    if ((h1=PAlloc(sizeof(EPChar)*(j+1)))==0) {
      ECoutErr ("[GetInterface] Can't create anchot point info...");
      return -1;
    }
    idlist=(EPChar*)PGet(h1);
    ii=j=0;

    for (i=0;i<Result+1;i++)
      if ((Buffer[i]==',')||(Buffer[i]==0)) {

	clist[j]=(Buffer+ii);
	Buffer[i]=0; /* Finish string at comma*/
	/*ECout1 ("[GetINterface] Substring for class %d: %s",j,clist[j]);*/
	if ((aux=strchr(clist[j],':'))!=NULL) {
	  idlist[j]=aux+1;
	  *aux=0;
	} else 
	  idlist[j]=NULL;	
	//ECout1 ("\n[GetInterface] Class %d : %s >%s<\n",j,clist[j],idlist[j]);
	j++;
	ii=i+1;

	}
    clist[j]=0;
    idlist[j]=0;
    gClase[CId].Status=CLASS_ILOADED;
    /*
     * At this point, interface is loaded and mapped for this process
     * but the shared seccion is only loaded
     */
    ProcMapTable[CId]=CLASS_IMAPPED;
    DeriveClass (gClase[CId].ClassName,clist,idlist);  
    SUnGet (clist);
    SUnGet (idlist);
    SFree (h,clist);
    SFree (h1,idlist);
  }
  CloseINI(pi);   
  return 0;
}


EUint32 EDMAPROC FreeInterface (EUint32 ClassId,EPChar Name)
{
  CLASSID		CId;
  
  /* We get the class identifier */
  if (ClassId==-1)
    CId=GetClassId (Name);
  else
    CId=ClassId;

  /* For all the process we must to deattach the shared memory block */
  SUnGet(pClase[CId].SCList);
  SUnGet(pClase[CId].SCIdList);
  SUnGet(pClase[CId].Prop);
  SUnGet(pClase[CId].Met);
  SUnGet(pClase[CId].Not);	   
  /* If no object uses this class we can free associated shared resources */
  ProcMapTable[CId]=CLASS_ILOADED;
 if (gClase[CId].Ocurrences==0)
   {
     SFree(pClase[CId].SysClass.hSCList,pClase[CId].SCList);
     SFree(pClase[CId].SysClass.hSCIdList,pClase[CId].SCIdList);
     SFree(pClase[CId].SysClass.hProp,pClase[CId].Prop);
     SFree(pClase[CId].SysClass.hMet,pClase[CId].Met);
     SFree(pClase[CId].SysClass.hNot,pClase[CId].Not);	 
     ProcMapTable[CId]=CLASS_DEF;
     gClase[CId].Status=CLASS_DEF;
   }

 return 0;
}


EUint32 LeePropiedad(PINIFILE pi,EUint32 iC,EUint32 iP)
{
  EChar		Buffer[200];
  EChar		Aux[100];
  EChar		Tipo[80];
  EDWord		Result;
  EUint32		i,size,Elem,IdC;
  
  size=0;
  sprintf (Aux,"Prop%d",iP);
  Result=GetINIString (pi,Aux,"Name",NULL,Buffer,200);
  strcpy(pClase[iC].Prop[iP].IdProp,Buffer);
  Result=GetINIString (pi,Aux,"Type",NULL,Buffer,200);
  for (i=0;i<MAX_TIPOS;i++)
    if (strcmp(tipo[i].Id,Buffer)==0) break;
  if (i==MAX_TIPOS)
    {
      ECoutErr ("[LeePropiedad] (%s)El tipo %s no est definido",Aux,Buffer);
    }
  pClase[iC].Prop[iP].Tipo=i;
  /* Special processing for user defined datatypes */
  if (i==DT_EUSER)
    {
      /* We get the datatype type */
      Result=GetINIString (pi,Aux,"UserType",NULL,Buffer,200);
      IdC=GetClassId(Buffer);
      if (IdC!=-1)
	pClase[iC].Prop[iP].UserInfo=(HMEM)IdC;
      else
	{
	  ECoutErr ("[LeePropiedad] Property %s declare %s user type not defined",pClase[iC].Prop[iP].IdProp,Buffer);
	  pClase[iC].Prop[iP].UserInfo=-1;
	}
      
    }
  Elem=GetINIInt (pi,Aux,"ArrayElems",0);
  pClase[iC].Prop[iP].nElem=Elem;
  size=tipo[i].tam;
  if (Elem!=0)
    size*=Elem;
  Result=GetINIString (pi,Aux,"Access",NULL,Buffer,200);
  
  if (strcmp(Buffer,"READ")==0)
    pClase[iC].Prop[iP].ioTipo=L;
  if (strcmp(Buffer,"WRITE")==0)
    pClase[iC].Prop[iP].ioTipo=E;
  if (strcmp(Buffer,"READ/WRITE")==0)
    pClase[iC].Prop[iP].ioTipo=E_L;
  
  return size;
}

EUint32 LeeMetodo(PINIFILE pi,EUint32 iC,EUint32 iP)
{
  EChar		Buffer[200];
  EChar		Aux[100];
  EChar		Tipo[80];
  EDWord	Result;
  EUint32	i,size;
  
  size=0;
  
  sprintf (Aux,"Met%d",iP);
  
  Result=GetINIString (pi,Aux,"Name",NULL,Buffer,200);
  strcpy(pClase[iC].Met[iP].IdMet,Buffer);

  /* We copy method's signature */
  Buffer[0]=0;
  Result=GetINIString (pi,Aux,"Signature",NULL,Buffer,200);
  strcpy (pClase[iC].Met[iP].Sign,Buffer);


  i=GetINIInt (pi,Aux,"Static",0);
  pClase[iC].Met[iP].Static=i;

  i=GetINIInt (pi,Aux,"Abstract",0);
  if (i) {
    pClase[iC].Met[iP].Virtual=i;
    pClase[iC].Met[iP].Abstract=i;
  }

  if (i==0) {
    i=GetINIInt (pi,Aux,"Virtual",0);
    pClase[iC].Met[iP].Virtual=i;
  }
  
  if (i)
    return 1;
  else
    return 0;
}


