/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

/*
*********************************************************
 Entorno de Desarrollo Modular y Abierto
 (c) David Martnez Oliveira
 Versin Beta 0.3r1
 Vigo 22 de Junio de 1997

 ---------------------------------------------------------

 Mdulo de funciones para herencia

 REVISIONES:------------------------------------------------
 25 de Abril de 1999
 Aadimos las funciones para el manejo de la tabla de subclases
 Reestructuracin del sistema de herencia dinmico.
 ------------------------------------------------
 * November, 18th, 2001
 * Code clean up
 */

#include <stdio.h>
#include <string.h>
#include "portable.h"
#include "classq.h"
#include "shmem.h"
#include "vglobal.h"
#include "clas.h"
#include "obj.h"
#include "misc.h"

/****************************************************************
** Dynamic Inheritance Functions
*******************************************************************/

/*
** Add super classes to an OBJECT
*/

EUint32 EDMAPROC AddSCObj1 (POBJ pObj,EPChar *c,EPChar *cId)
{
  EUint32	Id,IdSC,tam;
  EUint32	i,j,k;		/* General counters */ 
  EUint32	nSC;		/* Superclasses number*/
  EUint32	nSCOld;		/* Old Superclasses number*/
  EPChar	*cTemp;
  SC_ID		*NTabla;	/* To table clonning*/
  EChar		error[80];
  HMEM		hNTabla;
  EChar		ClassName[50];
  EChar         pap[50];
  EChar		SubSys[32];
  EPChar		Aux,Aux1,Aux2;
  EChar		isSIU;
  OBJID         new_obj;
  
  Id=pObj->IdClase;
  /* If class isn't loaded we load it*/
  if (ProcMapTable[Id]!=CLASS_LOADED)
    LoadClassImp (Id);
  
  /* We count superclasses */
  nSC=0;
  cTemp=c;
  while (cTemp[nSC]!=NULL)
    nSC++;
  
  /* We update superclass table and clone it */
  nSCOld=pObj->nTabla1;
  tam=sizeof(SC_ID)*(nSC+nSCOld);
  hNTabla=PAlloc(tam);
  if (hNTabla==0)  {
    ECoutErr ("Dynamical inheritance memory alloc errror");
    return -1;
  }
  
  NTabla=PGet(hNTabla);
  if (nSCOld)	/* Copy old classes if any*/
    memcpy (NTabla,pObj->Tabla1,nSCOld*sizeof(SC_ID));
  
  /* We delete old table*/
  PFree (pObj->SysObj.hTabla1,pObj->Tabla1);
  pObj->Tabla1=NTabla;
  pObj->SysObj.hTabla1=hNTabla;
  pObj->nTabla1+=nSC;
  /*.. and we fill in it*/
  for (i=0;i<nSC;i++) {
    /* Get class name */
    Aux=strchr(c[i],')');
    if (Aux!=NULL) {
      strncpy(SubSys,&c[i][0]+1,Aux-&c[i][0]-1);
      SubSys[Aux-&c[i][0]]=0;
      strcpy(ClassName,Aux+1);
      Aux++;
    }  else  { /* default system is EDMA*/
      strcpy(SubSys,"EDMA");
      strcpy(ClassName,c[i]);
      Aux=c[i];
    }
    ECoutDbg(4,-1,"(AddSC) Adding class %s in subsystem : %s",ClassName,SubSys);
    /*ECout1("[AddSC1] Adding class %s in subsystem : %s",ClassName,SubSys);*/
    if (strcmp(SubSys,"EDMA")==0) {
      isSIU=0;
      Aux1=strchr(c[i],':');
      if (Aux1!=NULL) {
	Aux1++;
	isSIU=1;
      } else Aux1=Aux;
#if 0      
    pap[0]=0;
    if ((Aux2=strchr (ClassName,'@'))!=NULL) {

      *Aux2=0;
      strcpy (pap,ClassName);
      //strcpy (ClassName,Aux+1);
      cId[i]=Aux2+1;
      c[i]=ClassName;
      ECout1 ("[AddSC1] PAP specified: %s %s-%s %s|",pap,ClassName,cId[i],c[i]);
    }
#endif

    //IdSC = GetClassId(Aux);
    IdSC = GetClassId(ClassName);
      if ((IdSC==GVar->nMaxClases)&&(isSIU==0)) {
	ECoutErr ("[AddSC1] Class %s not found",c[i]);
	/* FIXME: We need to rollback transaction*/
	return 0;
      }
      if ((ProcMapTable[IdSC]!=CLASS_LOADED)&&(isSIU==0))
	LoadClassImp (IdSC);
      
      if (isSIU==0)
	j=IdSC;
      else
	j=-1;
      pObj->Tabla1[nSCOld+i].IdClase=j;	     /* Class Identifier */
      
      strcpy (pObj->Tabla1[nSCOld+i].Id,cId[i]); /* User identifier */
      /* we create a new subobject*/
      /*pObj->Tabla1[nSCOld+i].Obj = &gObj[NewObj (ClassName,NULL)];	 */
     /* Modification building a new subobject. XXX: FIXME: Check for errors*/
      /* FIXME: This'd be moved to a new function*/
      new_obj=NewObj_BasicStage (ClassName,NULL);
      NewObj_InhStage(new_obj, pObj->IdObj,NULL);
      NewObj_VMStage(new_obj);
      NewObj_FinalStage (new_obj,NULL);
      pObj->Tabla1[nSCOld+i].Obj = &gObj[new_obj];
      /*----------------------------------------*/
      
      pObj->Tabla1[nSCOld+i].IdClase=j;	    /* Class identidfier*/
      /*ECout1 ("[AddSC1] Working on %s [%s] for class %d",c[i],cId[i],j);*/
      if (strcmp (c[i],cId[i])==0) {    /* If classname == cId -> on-demand inheritance*/
	/* We should look in SCIdList for the new created object*/
	/*ECout1 ("[AddSC1] Testing PAP...");*/
	j=gObj[new_obj].IdClase;
	if (pClase[j].SCIdList) { /* First test if there is a SCIdList for the object*/
	  for (k=0;k<gClase[j].Derived;k++)
	    if (pClase[j].SCList[k]==pObj->IdClase) 
	      break;
	  /*ECout1 ("[AddSC1] Index found : %d of %d",k,gClase[j].Derived);*/
	  if ((k==gClase[j].Derived) || (!pClase[j].SCIdList[k]))
	    strcpy (pObj->Tabla1[nSCOld+i].Id,cId[i]); /* User identifier*/
	  else
	    strcpy (pObj->Tabla1[nSCOld+i].Id,pClase[j].SCIdList[k]);
	}
      } else {
	/*ECout1 ("[AddSC1] Setting Anchor point to: %s",cId[i]);*/
	strcpy (pObj->Tabla1[nSCOld+i].Id,cId[i]); /* User identifier*/
      }

      gObj[((POBJ)pObj->Tabla1[nSCOld+i].Obj)->IdObj].Padre=pObj;
      
      gObj[((POBJ)pObj->Tabla1[nSCOld+i].Obj)->IdObj].PseudiPadre=pObj;
    } else { /* if is in SIU, superclass identifier is set to -1*/
      IdSC=-1;
      if (pObj->IdSIU==-1) {	/*if we don't have a SIU Proxy */
	ECoutErr("Cant inherit on SIU subsystem. Object %d isnt a SIU PROXY",pObj->IdObj);
	continue;
      }
      if (isSIU==0)
	j=IdSC;
      else
	j=-1;

      /*XXXXXXXXXXXXXXXXXXXXXXXXXXXx TODO XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX*/
      /* This code is for SIU Layer 2, and it isn't tested               */
      /*******************************************************************/
#if 0
      pObj->Tabla1[nSCOld+i].IdClase=j;	    /* Class identidfier*/
      ECout1 ("[AddSC1] Working on %s [%s]",c[i],cId[i]);
      if (strcmp (c[i],cId[i])==0) {    /* If classname == cId -> on-demand inheritance*/
	/* We should look in SCIdList for the new created object*/
	ECout1 ("[AddSC1] Testing PAP in object %d...",new_obj);
	j=gObj[new_obj].IdClase;
	if (pClase[j].SCIdList) { /* First test if there is a SCIdList for the object*/
	  ECout1 ("[AddSC1] Class %d has a SCIDList",j);
	  for (k=0;k<gClase[j].Derived;k++) {
	    ECout1 ("[AddSC1] Looking for PAP for %d. [%d] ",pObj->IdClase,k);//,pClase[j].SCList[k]);
	    if (pClase[j].SCList[k]==pObj->IdClase) 
	      break;
	  }
	  ECout1 ("[AddSC1] Index found : %d of %d",k,gClase[j].Derived);
	  if ((k==gClase[j].Derived) || (!pClase[j].SCIdList[k]))
	    strcpy (pObj->Tabla1[nSCOld+i].Id,cId[i]); /* User identifier*/
	  else
	    strcpy (pObj->Tabla1[nSCOld+i].Id,pClase[j].SCIdList[k]);
	}
      } else
	strcpy (pObj->Tabla1[nSCOld+i].Id,cId[i]); /* User identifier*/
#endif
      /*******************************************************************/
      pObj->Tabla1[nSCOld+i].Obj=pObj;
      Met3(pObj->IdObj,"AddSC",ClassName,cId[i]); /* SIU Proxy will create the subobject for us*/
    }
    
  }
  return 0;
}

/*
** Changes a class in an object
** This function changes superclasses alredy defined in the object
*/
/* December, 8th,2001: 
 FIXME: Test this function. We're using the same approach as in AddSC
*/
EUint32 EDMAPROC OverSCObj1 (POBJ pObj,EUint32 i,EPChar c,EPChar cId)
{
  EUint32	nSC;
  EUint32	Id,IdSC;
  EUint32	j,k;		/* General loop counters */
  EUint32	nSCOld;		/* Ols SuperClass number*/
  EPChar	*cTemp;
  SC_ID		*NTabla;	/* Used to table clonning*/
  HMEM		hNTabla;
  EChar		error[80];
  POBJ		OldObj,NObj;
  EChar		ClassName[50];
  EChar		SubSys[32];
  EChar         pap[50];
  EPChar	Aux,Aux1;
  EChar		isSIU;
  OBJID         new_obj;

  Aux=strchr(c,')');

  if (Aux!=NULL) {
    strncpy(SubSys,c,Aux-c-1);
    SubSys[Aux-c]=0;
    strcpy(ClassName,Aux+1);
    } else { /* by default we're in EDMA */
      strcpy(SubSys,"EDMA");
      strcpy(ClassName,c);
    }
  ECoutDbg(4,-1,"(OverSC) Adding class %s in subsystem : %s",ClassName,SubSys);
  /*ECout1("(OverSC) Adding class %s in subsystem : %s",ClassName,SubSys);*/

  if (strcmp(SubSys,"EDMA")==0) {
    isSIU=0;
    Aux1=strchr(c,':');
    if (Aux1!=NULL) {
      Aux1++;
      isSIU=1;
    } else 
      Aux1=Aux;

#if 0
    pap[0]=0;
    if ((Aux=strchr (ClassName,'@'))!=NULL) {

      *Aux=0;
      strcpy (pap,ClassName);
      //strcpy (ClassName,Aux+1);
      cId=Aux+1;
      c=ClassName;
      ECout1 ("[AddSC1] PAP specified: %s %s-%s",pap,ClassName,cId[i]);
    }
#endif

    IdSC = GetClassId(ClassName);
    if ((IdSC==GVar->nMaxClases)&&(isSIU==0)) {
      ECoutErr ("Class %s not found",c);
      return 0;
    }
    /*
    ECout1 ("[OverSC1] Creating object of class %s. Tabla1: %d [%s] ",ClassName,
	    ((POBJ)pObj->Tabla1[i].Obj)->IdObj,
	    gClase[pObj->IdClase].ClassName);
    ECout1 ("[OverSC1] %d",pObj->Tabla1[i].IdClase);
    */
    /* if it is the same we do nothing */
    if ((IdSC==pObj->Tabla1[i].IdClase)&&(isSIU==0))
      return 1;

    OldObj = pObj->Tabla1[i].Obj;

    /*
    ECout1 ("[OverSC1]-------------------");
    ECout1 ("[OverSC1] Working with Class IdSC : %d",IdSC);
    */
    if ((ProcMapTable[IdSC]!=CLASS_LOADED)&&(isSIU==0))
      LoadClassImp (IdSC);
    /*ECout1 ("[OverSC1] Working with Class IdSC : %d",IdSC);*/
    j=IdSC;

    /*NObj= &gObj[NewObj (gClase[IdSC].ClassName,NULL)];*/
    new_obj=NewObj_BasicStage (ClassName,NULL);
    NewObj_InhStage(new_obj, pObj->IdObj,NULL);
    NewObj_VMStage(new_obj);
    NewObj_FinalStage (new_obj,NULL);
    /*----------------------------------------*/

    NObj=&gObj[new_obj];
    /* XXX: Ooops... is this correct??????*/
    if (NObj==NULL) {
      ECoutErr("(OverSC1)Cant create subobject","Override SuperClass Error",0);
      return -1;
    }
    
    /*FreeObj (((POBJ)pObj->Tabla1[i].Obj)->IdObj);*/
    /*
    ECout1 ("[OvreSC1] %d -------------------------------------",i);
    ECout1 ("[OverSC1] Freeing subobject %d",((POBJ)pObj->Tabla1[i].Obj)->IdObj);
    */
    FreeOneObj (((POBJ)pObj->Tabla1[i].Obj)->IdObj);
    /*ECout1 ("[OverSC1] Freeing subobject %d",((POBJ)pObj->Tabla1[i].Obj)->IdObj);*/
    pObj->Tabla1[i].IdClase=j;	/* Class identifier*/

    /*ECout1 ("[OverSC1] Working on %s %s",c,cId);*/
    if (strcmp (c,cId)==0) {    /* If classname == cId -> on-demand inheritance*/
      /* We should look in SCIdList for the new created object*/
      /*ECout1 ("[OverSC1] Testing PAP...");*/
      j=gObj[new_obj].IdClase;
      if (pClase[j].SCIdList) { /* First test if there is a SCIdList for the object*/
	for (k=0;k<gClase[j].Derived;k++)
	  if (gObj[new_obj].Tabla[k].IdClase==pObj->IdClase) 
	    break;
	/*ECout1 ("[OverSC1] Index found : %d of %d",k,gClase[j].Derived);*/
	if ((k==gClase[j].Derived) || (!pClase[j].SCIdList[k]))
	  //strcpy (pObj->Tabla1[nSCOld+i].Id,cId); /* User identifier*/
	  strcpy (pObj->Tabla1[i].Id,cId); /* User identifier*/
	else
	  //strcpy (pObj->Tabla1[nSCOld+i].Id,pClase[j].SCIdList[k]);
	  strcpy (pObj->Tabla1[i].Id,pClase[j].SCIdList[k]);
	}
    } else
      strcpy (pObj->Tabla1[i].Id,cId);  /* User provided identifier*/
    
    /*ECout1 ("[OverSC1] Entry %d is : %s",i,pObj->Tabla1[i].Id);*/

    pObj->Tabla1[i].Obj = NObj;
    ((POBJ)pObj->Tabla1[i].Obj)->Padre=pObj;
    
    ((POBJ)pObj->Tabla1[i].Obj)->PseudiPadre=pObj;
  }
  else	{	/* if we override within SIU */
    Met3(pObj->IdObj,"OverSC",i,c,cId);
  }
  /*ECout1 ("[OverSC1] All done!");*/
  return 0;
}

/*
** General function for dynamic inheritance
*/

EUint32 EDMAPROC MutateObj1 (POBJ pObj,EPChar *c,EPChar *cId)
{
  EUint32	nSC;	       /* Superclasses counter*/
  EUint32	nOldSC;	       /* Old Superclasses counter*/
  EUint32	nNewSC;	       /* New Superclasses counter*/
  EUint32	i,j,k;	       /* general counter */
  EUint32	nTabla;	       /* Number of entries in object table*/
  EPChar	*cTemp;
  EPChar	*NewTabla;
  HMEM		hNewTala;
  EPChar	*NewIdTabla;
  HMEM		hNewIdTabla;
  EPUint32	IndTabla;
  HMEM		hIndTabla;
  POBJ		Obj,sObj;
  CLASSID       idC,thisId;
  EChar         pap[50],ClassName[50];
  EPChar        Aux2;
  
  /* We count the superclasses provided by user*/
  nSC=0;
  cTemp=c;
  while (cTemp[nSC]!=NULL)
    nSC++;

  nTabla=pObj->nTabla1;
  hIndTabla=PAlloc(sizeof(EUint32)*nSC);
  if (hIndTabla==(HMEM)0) {
    ECoutErr("(MutateObj1) Can't create SuperClass table for object %d",pObj->IdObj);
    return 0;
  }
  IndTabla=PGet(hIndTabla);
  /* FIXME: Test if alloc worked*/
  /* We find what superclasses are old*/
  nOldSC=0;
  for (i=0;i<nSC;i++) {
    IndTabla[i]=-1;
    /*
    pap[0]=0;
    strcpy (ClassName,c[i]);
    if ((Aux2=strchr (ClassName,'@'))!=NULL) {
      *Aux2=0;
      strcpy (pap,ClassName);
      //strcpy (ClassName,Aux+1);
      cId[i]=Aux2+1;
      c[i]=ClassName;
      ECout1 ("[AddSC1] PAP specified: %s %s-%s %s|",pap,ClassName,cId[i],c[i]);
    }
    */

    if (strcmp (c[i],cId[i])==0) { /* If classname==anchor point look for class id*/
      /*ECout1("[MutateObj1] Classname == Anchor point... looking for class id");*/
      idC=GetClassId(c[i]);
      if (idC==-1) {
	ECout1 ("[MutateObj1] Class %s doen't exist",c[i]);
	continue;
      }
      thisId=pObj->IdClase;
      /*ECout1 ("[MutateObj1] Class ID: %d %d",idC,thisId);*/
      for (j=0;j<nTabla;j++) {
	/*ECout1 ("[MutateObj1] Testing entry %d : %d",j,pObj->Tabla1[j].IdClase);*/
	sObj=(POBJ)(pObj->Tabla1[j].Obj);
	/*ECout1 ("[MutateObj1] Testing for %d Tabla entries in subobject %d",
	  sObj->nTabla,sObj->IdObj);*/
	/*if (pObj->Tabla1[j].IdClase==thisId) {*/
	for (k=0;k<sObj->nTabla;k++) {
	  /*ECout1 ("[MutateObj1] Testing %s [%d]",sObj->Tabla[k].Id,sObj->Tabla[k].IdClase);*/
	  if (sObj->Tabla[k].IdClase==thisId) {
	    IndTabla[i]=j;
	    nOldSC++;
	    break;
	  }
	}
      }
    } else   /* else look for anchor point*/
      for (j=0;j<nTabla;j++) {
	if (strcmp(pObj->Tabla1[j].Id,cId[i])==0) {
	  IndTabla[i]=j;
	  nOldSC++;
	  break;
	}
      }
  }
  /* We modify the old ones*/
  for (i=0;i<nSC;i++) {
    j=IndTabla[i];
    /*ECout1 ("[MutateObj] Overriding Subobject %d: %d",i,j);*/
    if (j!=-1) {
      OverSCObj1 (pObj,j,c[i],cId[i]);
    }
  }
  /* Build a table to store old superclasses*/
  nNewSC=nSC-nOldSC;
  /*ECout1("(MUtateObj1) Adding %d New Superclases",nNewSC);*/
  if (nNewSC) {	/* Are there some new superclass? */
    /*ECout1 ("[MutateObj1] There are new classes to create");*/
    j=0;
    for (i=0;i<nSC;i++)
      if (IndTabla[i]==-1) {
	c[j]=c[i];
	cId[j]=cId[i];
	j++;
      }
    PFree(hIndTabla,IndTabla);
    c[j]=NULL;
    cId[j]=NULL;
    AddSCObj1 (pObj,c,cId);
  }
  
  /* Look for virtual objects in upper level, and allow it to recalculate its tables*/
  /*
    Obj=pObj;
    while (Obj!=0)
    {
    if (Obj->Flag==VIRTUAL_OBJECT)
    MakeVirtual (Obj,0,1);
    Obj=Obj->Padre;
    }
  */
  /*ECout1 ("[MutateObj1] All done!");*/
  return 0;
}

/************************************************************************
** Object Merging functions *********************************************
**************************************************************************/

EUint32 EDMAPROC MergeObj1 (OBJID IdObj,EPChar Id,OBJID IdObj1)
{
  POBJ		pObj,pObj1;
  EUint32	nId,i,l,pos;
  EUint32	n,j;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		*PosStr;
  EChar		Error[80];
  EPChar	Id1;
  VIR_MET_PARAMS	VMParams;
  POBJ		ObjIni;
  PPROC         Func;   
  PMetVir	vMet;
  EUint32	tam;
  
  if ((IdObj<0)||(IdObj>GVar->nMaxObj)) {
    ECoutErr("(MergeObj) Object %d doesnt exist",IdObj);
    return 0;
  }
  if (gObj[IdObj].Flag==OBJ_FREE) {
    ECoutErr("(MergeObj) Object %d doesnt exist",IdObj);
    return 0;
  }
  
  if ((IdObj1<0)||(IdObj1>GVar->nMaxObj)) {
    ECoutErr("(MergeObj) Object %d doesnt exist",IdObj1);
    return 0;
  }
  if (gObj[IdObj1].Flag==OBJ_FREE) {
    ECoutErr("(MergeObj) Object %d doesnt exist",IdObj1);
    return 0;
  }
  if (IdObj1==0)
    ECout1("(MergeObj) Merging object 0 in object %d. Id %s",IdObj,Id);
  pObj=&gObj[IdObj];
  pObj1=&gObj[IdObj1];
  ObjIni=pObj;

  for (i=0;i<pObj->nTabla1;i++)
    if (strcmp(pObj->Tabla1[i].Id,Id)==0) break;
  ObjIni=pObj->Tabla1[i].Obj;
  
  if (i==pObj->nTabla1)	 { /* If no anchor point exists */
    ECoutErr("(MergeObj) Anchor point %s doesn't exist",Id);
    return 0;
  }
  ECoutDbg(4,-1,"Meging Object %d of class %s in table entry %d, Id %s",pObj1->IdObj,gClase[pObj1->IdClase].ClassName,i,Id);
  /* Test */
  ObjIni->Padre=NULL;
  /* Test ends */
  pObj->Tabla1[i].Obj=pObj1;
  pObj->Tabla1[i].IdClase=pObj1->IdClase;
  pObj1->Padre=pObj;
  pObj1->PseudiPadre=pObj;
  
  return 0;
}

/* Free entries in the superclass table */
EUint32 EDMAPROC FreeSC1 (OBJID IdObj,EPChar Id)
{
  POBJ		Obj;
  EUint32	i,j;
  
  Obj=&gObj[IdObj];
  /* Locate the superclass*/
  for (i=0;i<Obj->nTabla1;i++)
    if (strcmp(Obj->Tabla1[i].Id,Id)==0) break;
  if (i==Obj->nTabla1) {
    ECoutErr ("Anchor point %d doesn't exist",Id);
    return 1;
  }
  for (j=i;j<Obj->nTabla1-1;j++)
    memcpy (&Obj->Tabla1[j],&Obj->Tabla1[j+1],sizeof(SC_ID));
  
  Obj->nTabla1--;

  if (Obj->nTabla1<=0) {
    PFree (Obj->SysObj.hTabla1,Obj->Tabla1);
    Obj->nTabla1=0;
  }
  
  return 0;
}


