/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998,2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
 * Entorno de Desarrollo Modular y Abierto
 * Version 0.3r2
 * (c) David Martnez Oliveira
 * 
 * Module   : INIMAN
 * Type     : C Source file
 * Category : System dependency
 * Description :
 *    Simulates WIN32 API functios for managing Windows .INI files-like
 * 
 * Revisions:
 * 21/10/1997
 *   Creation
 *-----------------------------------------------------
 * Febraury, 7th, 2001
 * Code cleanup and comments translation
 * ------------------------------------------------------
 * November, 17th, 2001
 * Compile warnnings removal
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "portable.h"
#include "shmem.h"

#define INIMAN_C
typedef struct
{
   FILE    *f;
   EPChar   p;
   EUint32  tam;
   EChar    Name[500];
}INIFILE,*PINIFILE;

#include "iniman.h"


PINIFILE EDMAPROC OpenINI (EPChar Name)
{
  PINIFILE   pini;
  EUint32    tam,i;
  
  pini=(PINIFILE)PAlloc(sizeof(INIFILE));
  if (pini==NULL) {
    printf("\nCan't alloc INIFILE struct");
    return 0;
  }
  /* We open the file */
  strcpy(pini->Name,Name);
  pini->f=fopen(Name,"rb");
  if (pini->f==NULL) {
    printf ("\nCan't Open file %s",Name);
    return 0;
  }
  /* We read the whole file in memory */
  fseek (pini->f,0,SEEK_END);
  pini->tam=ftell (pini->f);
  pini->p=(EPChar)PAlloc(pini->tam);
  if (pini->p==NULL) {
    printf("\nCan't alloc %d bytes for file %s",pini->tam,Name);
    return 0;
  }
  memset (pini->p,0,pini->tam);
  fseek(pini->f,0,SEEK_SET);
  tam=fread (pini->p,sizeof(char),pini->tam,pini->f);
  
  fclose (pini->f);
  
  return pini;
}

EUint32 EDMAPROC CloseINI (PINIFILE pini)
{
  PFree((HMEM)pini->p,pini->p);
  PFree((HMEM)pini,pini);
  
  return 0;
}

EUint32 EDMAPROC GetINIInt(PINIFILE pini,EPChar Section,EPChar Val,EUint32 def)
{
  EPChar pi;
  EPChar pe;
  EChar  aux[1024];
  EUint32 i;
  
#ifdef WINAPI_32
  return GetPrivateProfileInt(Section,Val,def,pini->Name);
#endif
#ifdef LINUX
  pi=strstr (pini->p,Section); // localizamos seccion
  
  if (pi==NULL)
    return def;
  pe=strchr(pi,'[');
  if (pe!=NULL)
    *pe=0;  /* we get the interesting part of the string*/
  pi=strstr(pi,Val); /* We find field*/
  if (pi==NULL) {
    *pe='[';
    return def;
  }
  pi=strchr (pi,'='); /* We find value*/
  pi++;
  i=atoi(pi);
  *pe='[';
  return i;
#endif
}

EUint32 EDMAPROC GetINIString (PINIFILE pini,EPChar Section,EPChar Val,EPChar def,EPChar Buffer,EUint32 size)
{
  EPChar pi;
  EPChar pe,pe1;
  EChar  aux[80];
  EChar  sec[80];
  EUint32 i,n;
  
#ifdef WINAPI_32
  return GetPrivateProfileString(Section,Val,def,Buffer,size,pini->Name);
#endif
#ifdef LINUX
  memset (Buffer,0,size);
  strcpy (sec,"[");
  strcat (sec,Section);
  strcat (sec,"]");
  pi=strstr (pini->p,sec); /* We find secction */
  if (pi==NULL) {
    return 0;
  }
  pi++;
  pe=strchr(pi,'[');
  if (pe!=NULL)
    *pe=0;  /* We get the interesting part of the string */
  pi=strstr(pi,Val); /* We find field*/
  if ((pi==NULL)) {
    *pe='[';
    return 0;
  }
  
  pi=strchr (pi,'='); /* We find value */
  pe1=strchr(pi,13);
  if (pe1==NULL) {
    pe1=strchr(pi,10);
    if(pe1==NULL) {
      printf ("(GetINIString) Can't locate end of line");
      return 0;
    }
  }
  n=pe1-pi;
  n--;
  pi++;
  strncpy(Buffer,pi,n);
  Buffer[n]=0;
  *pe='[';
  
  n=strlen(Buffer);
  return n;
#endif
}

EUint32 EDMAPROC CleanINIString(EPChar Buffer)
{
  EUint32 i;
  
  for (i=0;i<strlen(Buffer);i++) {
    if(Buffer[i]=='\\') Buffer[i]='/';
  }
  return 0;
}

