/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
 * Febraury, 7th, 2001
 * Code cleanup and comment translation
 * --------------------------------------------------------
 * October, 14th,2001
 * Added PRealloc function
 * ---------------------------------------------------------
 * November, 18th, 2001
 * Now temporally file to get a uinique key are created in TMPDIR
 * a constant defined in conts.h
 * 
 * Compile warnnings removal
 */

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include "portable.h"
#include "const.h"

#include "shmem.h"
EChar   Buffer[1024];

HMEM EDMAPROC SAlloc (EUint32 size,EPChar n)
{
  EUint32   a,shmflg,f,i,mask;
  EChar     name[80];
  EPChar    p;
  key_t     k;
  
  /* We create a file for filemapping shared memory */
  /*strcpy (name,"/usr/local/EDMAtmp/");*/
  strcpy (name,TMPDIR);
  strcat (name,"/");
  strcat(name,n);
  f=open(name,O_WRONLY|O_CREAT|O_TRUNC);
  if (f==-1) {
      if (errno!=EEXIST) {
	perror("Create File");
	return -1;
      }
  }
  chmod(name,S_IRWXU|S_IRWXG|S_IRWXO);
  k=ftok(name,0);
  shmflg=0;
  
  a=shmget (k,size,IPC_CREAT|0777);
  if ((a==-1)&&(errno!=EEXIST)) {
    printf ("\n(SAlloc) ERROR in shared block : %s",name);
    printf ("\n ");
    perror("SAlloc(shmget):");
    return -1;
  }
  close(f);
  return ((HMEM)a);
}

EPVoid EDMAPROC SGet (HMEM a)
{
  EPVoid     p;
  EUint32    i;
  
  p=shmat((EUint32)a,0,SHM_RND);
  if ((int)p==-1) {
    perror("SGet :");
    return (EPVoid)-1;
  }
  
  return p;
}

/* Deattaches shared memory for process */
EPVoid EDMAPROC SUnGet (EPVoid p)
{
  EUint32    i;
  
  i=shmdt(p);
  if (i==-1) {
    perror("SUnGet :");
    return (EPVoid)-1;
  }
  
  return p;
}

void EDMAPROC SFree (HMEM i, EPVoid p)
{
  EUint32         a;
  struct shmid_ds buf;
  
  a=shmctl(i,IPC_RMID,&buf);
  if (a==1) {
    perror("SFree (IPC_RMID):");
    return;
  }
  /* Here we should delete the file */
}

/* Private memory management */
HMEM EDMAPROC PAlloc (EUint32 size)
{
  EPVoid p;
  EPChar t;
  EUint32 i;
  
  p=malloc(size);
  t=(EPChar)p;
  if (p!=NULL)
    for (i=0;i<size;i++) *(t+i)=0;
  return ((HMEM)p);
}

HMEM EDMAPROC PRealloc (HMEM a,EUint32 new_size) 
{
  a=(HMEM)realloc((EPVoid)a,new_size);

  return a;
}

EPVoid EDMAPROC PGet (HMEM a)
{
  return (EPVoid)a;
}

void  EDMAPROC PFree (HMEM a,EPVoid p)
{
  if (p==NULL)
    return;
  free(p);
}
      
