/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
*********************************************************
 Entorno de Desarrollo Modular y Abierto
 (c) David Martnez Oliveira
 Vigo 18 de Octubre de 1996

 Versin 0.3r1
 ---------------------------------------------------------

 Mdulo de funciones miscelnea
 
 --------------------------------------------------------
 14 de Noviembre de 1996
 (San Teleco 1:45 am)
 Vamos a aadir funciones para reservar bloques de memoria
 . Hacemos que los reserve EDMA, para probar si los problemas
 con los tamaos de los bloques al descargar clase es debido
 a esto
 ----------------------------------------------------------------
 29 de Abril de 1997
 Modificacin de tipos para Beta WIN32
 *********************************************************
 8 de Agosto de 1997
 Aadimos funciones para nombrar aplicaciones
 * -----------------------------
 * 19/11/1997
 *   Aadimos funciones para la gestion del tipo BUFFER
 *   Estas funciones sern movidas a otro fichero cuando alcancen
 * un nmero importante
 * ----------------------------------------------------
 * Febraury, 7th, 20001
 * Code cleanup and comment translation
 * -----------------------------------------------------------------
 * October, 14th,2001
 * Added EBufferRealloc function
 * -------------------------------------------------------------------
 * November, 18th, 2001
 * Compile Warnning removal
*/
#ifndef MISC_H
#define MISC_H
#include <stdarg.h>
#include "portable.h"
#include "misc.h"     
#include "error.h"
#include "vglobal.h"
#include "vserv.h"

/*
** Errors **********************************************
*/
#define NUM_ERRORS 17
#define NUM_TITLES 12


/* Datastruct for holding error info */
typedef struct
{
  EChar			ErrorText[50];
  EUint32		TitleIndex;
} EDMA_ERROR;

/* System Error Table */
EDMA_ERROR	message[NUM_ERRORS] = {
  {"Out of Memory for Class Table",0},
  {"Out of Memory for Object Table",0},
  {"Out of Memory for Application Table",0},
  {"Configuration File not Found",0},
  {"Aditional DLL management not implemented yet",1},
  {"Cant Create more Objects",2},
  {"Can't Add more Applications",3},
  {"Systems File not Found",4},
  {"Machine not Found",5},
  {"Operating System not Found",6},
  {"Unknow Operating System",0},
  {"Class Info Query Field undefined",7},
  {"Class SySInfo Query Field undefined",7},
  {"Can't Create more Classes",8},
  {"SuperClass Table Allocation Error",9},
  {"Object doesnt exist",10},
  {"Object isn't Virtual",11}};

EPChar		title[NUM_TITLES] = {
  "EDMA Inicialitation Error",
  "EDMA Implementation Message",
  "EDMA Object Creation Error",
  "EDMA Application Add Error",
  "EDMA Systems Startup Error",
  "EDMA Systems Management Error",
  "EDMA Query Class Error",
  "EDMA Class Creation Error",
  "EDMA Static Inheritance Error"
  "EDMA Primitive Error",
  "EDMA Property Overrride Error"};


EUint32 EDMAPROC ECout (EPChar a)
{
  printf ("%s\n",a);
  /*vDrawString (GVar->vServ,a);*/
  return 0;
}

EUint32 EDMAPROC ECout1(EPChar f,...)
{
  va_list	p;
  EChar	msg[200];
  
  va_start(p,f);
  vsprintf(msg,f,p);
  printf ("%s\n",msg);
  /*vDrawString (GVar->vServ,msg);*/
}

EUint32 EDMAPROC ECoutObj(OBJID IdObj,EPChar f,...)
{
  va_list	p;
  EChar	msg[200];
  EChar	msg1[200];
 
  if (IdObj>GVar->nMaxObj)
    return -1;
  if (gObj[IdObj].Flag==0)
    return -1;

  va_start(p,f);
  sprintf(msg1,"(%s)",gClase[gObj[IdObj].IdClase].ClassName);
  vsprintf(msg,f,p);
  strcat(msg1,msg);
  printf ("%s\n",msg1);
  /*vDrawString (GVar->vServ,msg1);*/
}

EUint32 EDMAPROC ECoutErr(EPChar f,...)
{
  va_list	p;
  EChar	msg[200];
  EChar	msg1[200];
  
  va_start(p,f);
  strcpy(msg1,"(ERROR)");
  vsprintf(msg,f,p);
  strcat(msg1,msg);
  printf ("%s\n",msg1);
  /*vDrawString (GVar->vServ,msg1);*/
}

EUint32 EDMAPROC ECoutDbg(EUint32 l,OBJID IdObj,EPChar f,...)
{
  va_list	p;
  EChar	msg[200];
  EChar	msg1[200];
  
  if (l>=GVar->DebugLevel) {
    va_start(p,f);
    if (IdObj==-1)
      strcpy (msg1,"(DEBUG-System)");
    else
      sprintf(msg1,"(DEBUG-%s)",gClase[gObj[IdObj].IdClase].ClassName);
    vsprintf(msg,f,p);
    strcat(msg1,msg);
    printf ("%s\n",msg1);
    /*vDrawString (GVar->vServ,msg1);*/
  }
}

/* Various functions... App Naming....*/
EUint32 EDMAPROC SetAppName (EPChar n)
{
  strcpy (AppName,n);
  return 0;
}

EPChar EDMAPROC GetSystemPath(EPChar c)
{
  strcpy (c,GVar->SystemPath);
  return 0;
}

EUint32 EDMAPROC SetDebugLevel (EUint32 l)
{
  EUint32		o;
  
  o=GVar->DebugLevel;
  GVar->DebugLevel=l;
  return 0;
}

/*
 * Funciones de Gestin del tipo EDMAT_BUFFER
 * 19/11/1997
 * ------------------------
 * Estas funciones compondrn un nuevo mdulo del sistema cuando
 * alcancen un nmero importante
 * ---------------------------------------------------------------
 * Febraury, 7th, 2001
 * EDMAT_BUFFER datatype management functions
 */

ESint32 EDMAPROC EBufferAlloc (EDMAT_BUFFER *Buf,EUint32 Size)
{
  Buf->h=PAlloc(Size);
  if (Buf->h==0) {
    ECoutErr ("(EBufferAlloc) Can't allocate buffer");
    return 1;
  }
  Buf->dat=(EPVoid)PGet(Buf->h);
  Buf->Size=Size;
  /*We zeroed memory block*/
  memset(Buf->dat,0,Size);
  return 0;
}

ESint32 EDMAPROC EBufferFree (EDMAT_BUFFER *Buf)
{
  if (Buf->Size!=0)
    PFree(Buf->h,Buf->dat);
  Buf->Size=0;
  return 0;
}

ESint32 EDMAPROC EBufferRealloc (EDMAT_BUFFER *Buf,EUint32 Size)
{
  EUint32    old_size;

  /* CAUTION!!!: Realloc function had not been developed with windows
   32 compatibility in mind... that is, we don't know what's the windows
  API for this, so this interface can change in future*/
  Buf->h=PRealloc (Buf->h,Size);
  if (Buf->h==0)  {
    ECoutErr ("(EBufferRealloc) Can't reallocate buffer");
    return -1;
  }
  return 0;
}

#endif
