/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
 Entorno de Desarrollo Modular y Abierto
 Versin Beta 0.3r1
 (c) David Martnez Oliveira

 Modulo para gestin de multiples IDFs
 Revisiones: ---------------------------------------------------------
 24 de Julio de 1997
 -----------------------------
 24 de Agosto de 1997
 Intentamos aadir las funciones de soporte para la creacin
 de parsers IDF
 *---------------------------------------------------
 * Febraury, 7th, 2001
 * Code cleanup and comment translation
 * --------------------------------------------------------
 * November, 17th, 2001
 * Updated IDFSetMet to set abstract and static flags
 * ------------------------------------------------------------
 * November, 18th, 2001
 * Compile Warnnings removal
 * --------------------------------------------
 * November, 20th, 2001
 * Added new primitives GetNParsers and GetParserClass
 */

#include <stdio.h>
#include "portable.h"
#include "multiidf.h"
#include "vglobal.h"
#include "classq.h"
#include "misc.h"
#include "inh.h"

ESint32 EDMAPROC GetNParsers (void) {
  return GVar->nIdf;
}

CLASSID EDMAPROC GetParserClass (ESint32 i) {
  if ((i<0) || (i>GVar->nIdf))
    return -1;

  return GVar->IDFParser[i].IdClass;
      
}

ESint32 EDMAPROC GetParser (EPChar Name)
{
  EUint32		i;
  
  for (i=0;i<GVar->nIdf;i++) {
    if (strcmp(GVar->IDFParser[i].Id,Name)==0)
      break;
  }
  if (i==GVar->nIdf) 
    i=-1;
  ECout1 ("Parser id returned : %d",i);
  return i;
}

EUint32 EDMAPROC AddParser (EPChar Name,EPChar ClassName)
{
  ESint32	IdParser;
  CLASSID	IdClass;
  EChar		error[80];

  /*We check if the parser already exists*/
  /*ECout1 ("Adding new IDF parser : %s. Currently there are %d parsers",Name,GVar->nIdf);*/
  IdParser=GetParser(Name);
  if (IdParser!=-1) {
    ECoutErr ("IDF Parser %s already exists",Name);
    return 0;
  }
  /*... if it doen'r exists we add it */
  /*ECout1 ("Looking for Class : %s.",ClassName);*/
  IdClass=GetClassId(ClassName);
  /*ECout1("IDF Parser Class : %s %d. Parser id: %d",ClassName,IdClass,GVar->nIdf+1);*/
  IdParser=GVar->nIdf;
  strcpy (GVar->IDFParser[IdParser].Id,Name);
  GVar->IDFParser[IdParser].IdClass=IdClass;
  GVar->nIdf++;
  return 0;
}

/*
 * IDF Parser developing support functions
 */

/* Get a valid class identifier */
CLASSID EDMAPROC IDFGetClassID()
{
  EUint32		i;
  
  for (i=0;i<GVar->nMaxClases;i++)
    if (gClase[i].Status==CLASS_FREE)
      break;
  
  if (i==GVar->nMaxClases) {
    ECoutErr ("Cant alloc class identifier");
    return -1;
  }  else {
    gClase[i].Status=CLASS_TEMP;
    return i;
  }
}

ESint32 EDMAPROC IDFSetGeneral(CLASSID Id,EPChar Name,EPChar SO,EPChar Maq,EPChar Module)
{
  if (GetClassId(Name)==-1)
    return -1;
  strcpy (gClase[Id].ClassName,Name);
  gClase[Id].SOId=GetSOId(SO);
  gClase[Id].MaqId=GetMaqId(Maq);
  strcpy(gClase[Id].SysClass.ModuleName,Module);
  
  return 0;
}


ESint32 EDMAPROC IDFSetDef(CLASSID CId,EUint32 nProp,EUint32 nMet,EUint32 nNot)
{
  HMEM		h;
  EChar		Aux[80];
  
  gClase[CId].nProp=nProp;
  gClase[CId].nMet=nMet;
  gClase[CId].nNot=nNot;
  
  if(nProp!=0) {
    strcpy (Aux,gClase[CId].ClassName);
    strcat (Aux,"Prop");
    
    h=SAlloc(nProp*sizeof(PROP),Aux);
    if (h==(HMEM)0) {
      ECoutErr ("Can't allocate propieties table");
      return -1;
    }
    
    pClase[CId].Prop=(PROP*)SGet(h);
    pClase[CId].SysClass.hProp=h;
  }
  
  strcpy (Aux,gClase[CId].ClassName);
  strcat (Aux,"Met");
  if (nMet!=0) {
    h=SAlloc(nMet*sizeof(MET),Aux);
    if (h==(HMEM)0)
      {
	ECoutErr("Can't allocate method table");
	return -1;
      }
    
    pClase[CId].Met=(MET *)SGet(h);
    pClase[CId].SysClass.hMet=h;
    
    /* Per-process code mapping */
    
    fClase[CId].hFMet=PAlloc(sizeof(PROCMET)*nMet);
    if (fClase[CId].hFMet==0) {
      ECoutErr ("Cant allocate process method table");
      return -1;
    }
    fClase[CId].Met=(PROCMET*)PGet(fClase[CId].hFMet);
    
  }
  strcpy (Aux,gClase[CId].ClassName);
  strcat (Aux,"Not");
  if (nNot!=0) {
    h=SAlloc(nNot*sizeof(NOT),Aux);
    if (h==(HMEM)0) {
      ECoutErr ("Can't allocate notification table");
      return -1;
    }
    
    pClase[CId].Not=(NOT*)SGet(h);
    pClase[CId].SysClass.hNot=h;
  }
  return 0;
}

ESint32 EDMAPROC IDFSetProp (CLASSID iC,EUint32 iP,EPChar Name,EPChar pType,EPChar pAc,EUint32 nElem)
{
  ESint32       size;
  EUint32	i;
  
  strcpy(pClase[iC].Prop[iP].IdProp,Name);
  
  for (i=0;i<MAX_TIPOS;i++)
    if (strcmp(tipo[i].Id,pType)==0) break;
  if (i==MAX_TIPOS) {
    ECoutErr ("El tipo %s no est definido",pType);
  }
  pClase[iC].Prop[iP].Tipo=i;
  
  pClase[iC].Prop[iP].nElem=nElem;
  size=tipo[i].tam;
  if (nElem!=0)
    size*=nElem;
  
  if (strcmp(pAc,"READ")==0)
    pClase[iC].Prop[iP].ioTipo=L;
  if (strcmp(pAc,"WRITE")==0)
    pClase[iC].Prop[iP].ioTipo=E;
  if (strcmp(pAc,"READ/WRITE")==0)
    pClase[iC].Prop[iP].ioTipo=E_L;
  
  return size;
}

ESint32 EDMAPROC IDFSetMet (CLASSID iC,EUint32 iP,EPChar mName,EPChar Sig,EByte vFlag, EByte aFlag, EByte sFlag)
{
  
  strcpy(pClase[iC].Met[iP].IdMet,mName);
  strcpy (pClase[iC].Met[iP].Sign,Sig);
  pClase[iC].Met[iP].Virtual=vFlag;
  pClase[iC].Met[iP].Abstract=aFlag;
  pClase[iC].Met[iP].Static=sFlag;
  
  if (vFlag)
    return 1;
  else
    return 0;
  
}

ESint32 EDMAPROC IDFSetSCList (CLASSID Id,EPChar	*IdSC)
{
  if (IdSC!=NULL)
    DeriveClass(gClase[Id].ClassName,IdSC,NULL);
}

ESint32 EDMAPROC IDFSetClassID (CLASSID Id)
{
  EUint32		i,size,size1;
  
  /* We calculate data block size*/
  size=0;
  for (i=0;i<gClase[Id].nProp;i++) {
    size1=tipo[pClase[Id].Prop[i].Tipo].tam;
    if (pClase[Id].Prop[i].nElem!=0)
      size1*=pClase[Id].Prop[i].nElem;
    size+=size1;
  }
  gClase[Id].TamDatos=size;
  /* We count virtual methods*/
  size=0;
  for (i=0;i<gClase[Id].nMet;i++)
    size+=pClase[Id].Met[i].Virtual;
  gClase[Id].nMetVir=size;
  
  gClase[Id].Status=CLASS_ILOADED;
  ProcMapTable[Id]=CLASS_IMAPPED;
  return 0;
}
