/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
 Entorno de Desarrollo Modular y Abierto
 (c) David Martnez Oliveira
 Versin Beta 0.3r1
 15 de Junio de 1997

 REVISIONES:-------------------------------------------------------------
 15 de Junio de 1997
 Este mdulo incorpora las funciones bsicas para la gestin de
 objetos.

 Tenemos que mapear las DLLs que contienen la implementacin de las clases
 en el espacio de direccionamiento del proceso que invoca las primitivas de
 creacin de objetos. A la fecha de hoy simplemente hacemos un "LoadLibrary"
 cada vez que se crea un objeto, pero esto slo tenemos que hacerlo por cada
 proceso que se mapea en EDMA32.DLL.

 Con las clases de STOCK no debera haber problema, puesto que es el mdulo
 de inicializacin principal el que las carga en memoria y las mapea a cada
 proceso cada vez que carga EDMA.DLL. Una clase definida dinmicamente solo
 sera accesible al proceso que la cre, aunque todos puedan verla (tendran
 que realizar el mapeo a mano)...
------------------------------------------------------------------------------
 7 de Julio de 1997
 Aadimos soporte para herencia esttica si se han modificado las
 funciones de creacin de objetos para construir las estructuras adecuadas.
 Se siguen utilizando punteros a los objetos internamente en algunas funciones
 por comodidad y rapidez, pero ahora los subobjetos se crean en la matriz del sistema
 permitiendo mayor control para el "Garbaje Collection"
 ----------------------------------------
 13 de Julio de 1997
 Movemos GetPObj al mdulo de QUery de objetos
 ----------------------------------
 19 de Julio de 1997
 Aadimos soporte para el mapeo de las clases para cada proceso que lo necesite
 y solo cuando lo necesite
 --------------------------------------------------
 28 de Julio de 1997
 Modificacin para soporte de carga dinmica de implementacin de clases
 bajo de manda
 -------------------------------------
 7 de Agosto de 1997
 Aadimos soporte para ejecucin de constructores y destructores.
 Los contructores todava no soportan polimorfismo.
 --------------------------------------
 2 de Agosto de 1997
 Hemos aadido el modificador final a los objetos que impide la herencia bajo
 demanda.
 Vamos a intentar modificar NewObj para indicar el subsistema del objeto
 en el nombre de la clase.
 * -----------------------
 * 18 de Noviembre de 1997
 *    Modificaciones para soporte de tipos definidos por el usuario
 * Durante la creacin del objeto, debemos crear un objeto por cada tipo
 * definido por el usuario
 * -------------------------------------
 * 10th May 1999
 *    Aadimos informacin de subclases para conocer el camino de vuelta 
 * en las relaciones de herencia esttica.
 *-----------------------------------------------------------
 * Febraury, 7th, 2001
 * Code cleanup and comment translation
 * ---------------------------------------------------------------
 * November, 17th, 2001
 * Compile warnnings removal
 * -----------------------------------------------------------------
 * December, 3rd, 2001
 * Modification to use the preferer anchor points for static inheritance
 * when creating an object
 * -----------------------------------------------------------------------
 * December, 6th, 2001
 * Modification to make build objects process a mutistage procedure
 * We need this to allow dynamic inheritance functions build class
 * hierarchies correctly
 * -----------------------------------------------------------------------------
 * December, 8th, 2001
 * Updated NewObj_VMStage to autoimagically override homonym virtual methods
*/

#include <stdio.h>
#include <string.h>
#include "portable.h"
#include "tobj.h"
#include "vglobal.h"
#include "classq.h"
#include "obj.h"

EUint32 FreeVirData (POBJ,VIROBJ_DATA *);



OBJID EDMAPROC NewObj_BasicStage (EPChar ClassNam1, EPVoid ConstPar) {
  EUint32	i,j,idx;
  CLASSID	IdC,IdC1;
  OBJID		IdO;
  EChar		Mensaxe[80],Cadena[80];
  HMEM		h;
  ESint32	isSIU;
  EPChar	ClassName;
  EPChar	SubSys;
  EChar         Temp[80];   
  
  /* We see if the class identifier names a SIU subsystem*/
  strcpy(Temp,ClassNam1);   
  SubSys=strchr(Temp,':');
  
  if (SubSys!=NULL) {
    ClassName=SubSys+1;
    *SubSys=0;
    SubSys=Temp;
    ECout1("New object in system (%s) of class %s",SubSys,ClassName);
    IdC=GetClassId(SubSys);
    if (IdC==-1) {
      ECoutErr ("Class %s doesnt exist",ClassName);
      return -1;
    }
    isSIU=GetProxy(SubSys);
  }  else {    /* We locate the class */
    ClassName=ClassNam1;
    isSIU=-1;
    IdC=GetClassId(ClassName);
    if (IdC==-1) {
      ECoutErr ("Class %s doesnt exist",ClassName);
      return -1;
    }
    
    if (gClase[IdC].SIUProxy!=0) {
      isSIU=IdC;
      /* if the is in SIU we must create a SIU proxy for the EDMA side*/
      IdC=GVar->SIUProxy[gClase[IdC].SIUProxy].IdClass;
      ECout1 ("New object in SIU subsystem. Using SIU Proxy %s",gClase[IdC].ClassName);
    }
  }
  /* We map the library which creates the object for this process */
  if (ProcMapTable[IdC]!=CLASS_LOADED)
    LoadClassImp(IdC);
  
  /* We get a object identifier*/
  for (i=0;i<GVar->nMaxObj;i++)
    if (gObj[i].Flag==OBJ_FREE)
      break;
  if (i==MAX_OBJ) {
    ECoutErr ("Can't create more objects");
    return -1;
  }
  IdO=i;
  /* We create a data block for this object*/
  
  sprintf (Cadena,"%dData",IdO);
  h=PAlloc(gClase[IdC].TamDatos);
  if (h==(HMEM)0) {
    ECoutErr("Can't create local storage for Object");
    return -1;
  }
  gObj[IdO].SysObj.hDatos=h;
  gObj[IdO].Datos=(EPVoid)PGet(h);
  
  /* Here, the object has benn physically created */
  /* We setup general information for it */
  gObj[IdO].IdObj=IdO;
  gObj[IdO].IdClase=IdC;
  gObj[IdO].IdApp=AppId;
  gObj[IdO].Padre=NULL;
  gObj[IdO].PseudiPadre=NULL;
  gObj[IdO].Flag=OBJ_EXIST;
  gObj[IdO].IdSIU=isSIU;
  gObj[IdO].Final=0;
  /* Modificacin 12 de Noviembre de 1997. Problema en Linux */
  gObj[IdO].VirData.vMet=0;
  gObj[IdO].VirData.vProp=0;
  /* Fin Modificacion */
  
  GVar->nObj++;
  gObj[IdO].nTabla=0;

  return IdO;
}

/*************************************************************************
 * XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 * XXX: This function should be moved to other file.....
 * XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
 ****************************************************************************/
/*********************************************************************************/
ESint32 EDMAPROC Look4MetUp (POBJ *pObj1,EPChar MetName)
{
  EUint32     i,j;
  CLASSID     Tipo;
  ESint32     Flag;
  POBJ        pObj,pObj2;
  
  
  pObj=*pObj1;
  for (j=0;j<pObj->nTabla;j++) {
    pObj2=pObj->Tabla[j].Obj;
    Flag=Look4Met(&pObj2,MetName);
    if (Flag!=-1) {
      *pObj1=pObj2;
    }
    return Flag;
  }
}
/*******************************************************************************/

/*----------------------------------------------------------------------------*/
OBJID EDMAPROC NewObj_VMStage (OBJID IdO) {
  EUint32	i,j,idx;
  CLASSID	IdC,IdC1;
  EChar		Mensaxe[80],Cadena[80];
  HMEM		h;
  ESint32	isSIU;
  EPChar	ClassName;
  EPChar	SubSys;
  EChar         Temp[80]; 
  POBJ          pObj;

  IdC=gObj[IdO].IdClase;
  /* First we must to alloc memory for virtual methods */
  if (gClase[IdC].nMetVir) {
    gObj[IdO].SysObj.hvTabla=PAlloc(gClase[IdC].nMetVir*sizeof(PMetVir));
    gObj[IdO].vTabla=(PMetVir*)PGet(gObj[IdO].SysObj.hvTabla);
    if (gObj[IdO].SysObj.hvTabla==0) {
      ECoutErr("ERROR**************Can't Allocate Virtual Table");
      return -1;
    }
    j=0;
    /*ECout1 ("(NewOBJ_VMStage) Setting VMets for object %d [%s]",IdO,gClase[IdC].ClassName);*/
    for (i=0;i<gClase[IdC].nMet;i++) {
      /*ECout1 ("[NewObj_VMStage] Testing Method : %s",pClase[IdC].Met[i].IdMet,
	pClase[IdC].Met[i].Virtual);*/
      /* Look for homonym methods up in class hierarchy*/
      pObj=&gObj[IdO];
      if ((Look4MetUp(&pObj,pClase[IdC].Met[i].IdMet))!=-1) {
	if (pObj->IdObj!=IdO)
	  if (pClase[pObj->IdClase].Met[i].Virtual) {
	    /*	    
	    ECout1 ("[NewObj_VMStage] Overriding Method %s of object %d [%s] with ",
		    pClase[IdC].Met[i].IdMet,pObj->IdObj, gClase[pObj->IdClase].ClassName);
	    ECout1 ("[NewObj_VMStage] method %s of object %d [%s]",
		    pClase[IdC].Met[i].IdMet,IdO,gClase[IdC].ClassName);
	    */
	    OverMet1 (pObj->IdObj, pClase[IdC].Met[i].IdMet,
		      &gObj[IdO],(PPROC)fClase[IdC].Met[i].Func,pClase[IdC].Met[i].IdMet);
	  }
      }
      if (pClase[IdC].Met[i].Virtual) {
	gObj[IdO].vTabla[j].Ind=i;
	strcpy ((char*)gObj[IdO].vTabla[j].Id,pClase[IdC].Met[i].IdMet);
	gObj[IdO].vTabla[j].Func=(PPROC)fClase[IdC].Met[i].Func;
	gObj[IdO].vTabla[j].Obj=&gObj[IdO];
	gObj[IdO].vTabla[j].next=NULL;
	j++;
      }
    }
  }
}

/*---------------------------------------------------------------------------*/
/* FIXME: Modify this code to link current object with the Father one passed 
 * as a parameter. This will be used from dynamic inheritance prmitives.
 *
 * Probably we'll need to pass a OBJID vector for this function              */
OBJID EDMAPROC NewObj_InhStage (OBJID IdO, OBJID Father,EPVoid ConstPar) {
  EUint32	i,j,idx;
  CLASSID	IdC,IdC1;
  OBJID         id;
  EChar		Mensaxe[80],Cadena[80];
  HMEM		h;
  ESint32	isSIU;
  EPChar	ClassName;
  EPChar	SubSys;
  EChar         Temp[80]; 
  ESint32       flag=0;

  IdC=gObj[IdO].IdClase;
  ClassName=gClase[IdC].ClassName;
  /* If the object's class is derived... we create subobjects */
  if (gClase[IdC].Derived) {
    /* We build a table for storing subobject information */
    ECoutDbg (4,-1,"(NewObj)Object have %d superclases",gClase[IdC].Derived);
    gObj[IdO].nTabla=gClase[IdC].Derived;
    gObj[IdO].SysObj.hTabla=PAlloc(gClase[IdC].Derived*sizeof(SC_ID));
    gObj[IdO].Tabla=(SC_ID*)PGet(gObj[IdO].SysObj.hTabla);
    if (gObj[IdO].SysObj.hTabla==0) {
      ECoutErr ("Can't create SC table for object");
      return -1;
    }
    for (i=0;i<gClase[IdC].Derived;i++) {
      /*We create the subobject*/
      ECoutDbg (4,-1,"(NewObj)Creating superobject of class %s",gClase[pClase[IdC].SCList[i]].ClassName);
      /*ECout1 ("(NewObj_InhStage)Creating superobject of class %s",gClase[pClase[IdC].SCList[i]].ClassName);*/
      /*
	if (Father) {
	if (Father[i]!=-1)
	id=Father[i];
	} else
	id=NewObj(gClase[pClase[IdC].SCList[i]].ClassName,ConstPar);
      */
      /* If father parameter is in the object's class SCList, use the father parameter
       instead of create a new object */
      if ((Father!=-1) && (gObj[Father].IdClase == pClase[IdC].SCList[i])) {
	/*ECout1 ("(NewObj_InhStage] There is a father for this object...");*/
	id=Father;
	flag=1;
      } else {
	/*ECout1 ("No father for this object... creating it");*/
 	id=NewObj(gClase[pClase[IdC].SCList[i]].ClassName,ConstPar);
      }
      
      gObj[IdO].Tabla[i].Obj=&gObj[id];
      strcpy(gObj[IdO].Tabla[i].Id,gClase[pClase[IdC].SCList[i]].ClassName);
      gObj[IdO].Tabla[i].IdClase=pClase[IdC].SCList[i];

      if (!flag) {
	/*ECout1 ("No Father provided so we must to link it");*/
	
	/**** XXXX: Test this.... *****/
	gObj[id].Padre=&gObj[IdO];
	/* Modificacin 10 mayo 1999
	 * Como es una superclase, tiene que tener al menos una entrada
	 * en la tabla de subclases
	 */
	ECoutDbg (4,-1,"(NewObj)Building subclass info. Creating subclass table");
	
	/* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	   FIXME: What if a superclass has other subclases... it'll have
	   entries in its subclase table when the subobject gets created.
	   
	   The code here, will destroy the first suclass entry
	   Use AddSC1 instead of the code bellow
	   --------------------------------
	   There is no static inheritance information about subclasses
	   so 0 is the right entry when a new subobject is created
	   XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	*/
	gObj[id].nTabla1=1;
	gObj[id].SysObj.hTabla1=PAlloc(1*sizeof(SC_ID));
	if (gObj[id].SysObj.hTabla1==0) {
	  ECoutErr ("Can't create SubClass table for object");
	  return -1;
	}
	gObj[id].Tabla1=(SC_ID*)PGet(gObj[id].SysObj.hTabla1);
	ECoutDbg(4,-1,"(newObj)Subclass table succesfully created");
	/* We have one entry in the object's subclass table
	 * now, we must to fill the entry 0 with the subclass info*/
	/*if (!flag) { */
	gObj[id].Tabla1[0].Obj=&gObj[IdO];
	
	/*------------------------------*/
	idx=gObj[IdO].IdClase;
	if (pClase[idx].SCIdList)
	  if (pClase[idx].SCIdList[i]) {
	    /*ECout1 ("[NewObj] Preferere Anchor point for %s : %s",
	      ClassName,pClase[idx].SCIdList[i]);*/
	    strcpy(gObj[id].Tabla1[0].Id,pClase[idx].SCIdList[i]);
	  } else
	    strcpy(gObj[id].Tabla1[0].Id,ClassName);
	/*------------------------------*/
	/* FIXME: In the PAP case we'd override the homonym virtual methods
	 * in the superclases in order to make the structure consistent*/
	ECoutDbg (4,-1,"(NewObj)ClassName (%s) initiated",ClassName);
	gObj[id].Tabla1[0].IdClase=IdC;
	ECoutDbg(4,-1,"(NewObj)Subclass Info attached");
	/* FIn modificacin 10 de Mayo 1999 */
      }
    }
  }

  return IdO;
}

/*---------------------------------------------------------------------------------------------*/
OBJID EDMAPROC NewObj_FinalStage (OBJID IdO,EPVoid ConstPar) {
  EUint32	i,j,idx;
  CLASSID	IdC,IdC1;
  EChar		Mensaxe[80],Cadena[80];
  HMEM		h;
  ESint32	isSIU;
  EPChar	ClassName;
  EPChar	SubSys;
  EChar         Temp[80]; 

  IdC=gObj[IdO].IdClase;
  isSIU=gObj[IdO].IdSIU;
  /* Here, we execute the constructor */
  for (i=0;i<gClase[IdC].nMet;i++)
    if (strcmp(pClase[IdC].Met[i].IdMet,"born")==0)
      break;
  
  if (i!=gClase[IdC].nMet)	/* if we have got one */
    Met1 (IdO,i,ConstPar);
  
  /* If the object is external using SIU, we invoke NewObj primitive
   * in the SIU proxy object
   */
  if (isSIU!=-1) {
    gObj[IdO].IdSIU=isSIU;
    ECoutDbg (4,-1,"Invoking SIU Proxy NewObj....");
    Met3(IdO,"NewObj",ClassName);
  }
  
  ECoutDbg (4,-1,"(%s) Creates Object %u of Class %s",AppName,IdO,ClassName);
  ECoutDbg (4,-1,"(NewObj)Object of class %s created",ClassName);
  gClase[gObj[IdO].IdClase].Used=1;
  /* User datatypes extension... not fully tested */
  for (i=0;i<gClase[IdC].nProp;i++)
    if (pClase[IdC].Prop[i].Tipo==DT_EUSER) {
      IdC1=pClase[IdC].Prop[i].UserInfo;  /* Get datatype class*/
      j=(OBJID)NewObj(gClase[IdC1].ClassName,NULL); /* Create object*/
      /*We introduce the new object in the right place*/
      *((OBJID*)((EPByte)gObj[IdO].Datos+pClase[IdC].Prop[i].Off))=j;
    }
  return IdO;
}

/* The top-level NewObj Primitive. It doesn't change in order to
   don't modify availables apps*/
/* FIXME: To a cleanup on error:
 *        Free object idenfier in object table
 *        Free object associated memory  
 */
OBJID EDMAPROC NewObj (EPChar ClassNam1,EPVoid ConstPar) {
  OBJID              id;

  /* First the basic stage. This allocs minimum memory requiremnts
     as well as, it does basic initialization*/
  /*ECout1 ("[NewObj] Basic Stage....");*/
  if ((id=NewObj_BasicStage(ClassNam1,ConstPar))==-1){
    ECoutErr ("[NewObj] Basic Stage failed. Can't create new Object");
    return -1;
  }
  /* Then, process the static inheritance information */
  /*ECout1 ("[NewObj] Inheritance Stage (%d)",id);*/
  if ((NewObj_InhStage(id,-1,ConstPar))==-1) {
    ECoutErr ("[NewObj] Inheritance Stage failed. Can't create Object");
    return -1;
  }
  /* Then the virtual method management*/
  /*ECout1 ("[NewObj] VM Stage (%d)",id);*/
  if ((NewObj_VMStage(id))==-1) {
    ECoutErr ("[NewObj] Virtual Method Stage failed. Can't create Object");
    return -1;
  }

  /* Last, run constructor if any*/
  /*ECout1 ("[NewObj] Latest Stage (%d)",id);*/
  if ((NewObj_FinalStage(id,ConstPar))==-1){
    ECoutErr ("[NewObj] Final Stage Failed. Can't create Object");
    return -1;
  }
  /*ECout1 ("[NewObj] All done (%d)!!!",id);*/
  return id;
}











/*********************************************************************************************/
/*********************************************************************************************/
/*********************************************************************************************/
/*********************************************************************************************/
#if 0
ESint32	EDMAPROC NewObj (EPChar ClassNam1,EPVoid ConstPar)
{
  EUint32	i,j,idx;
  CLASSID	IdC,IdC1;
  OBJID		IdO;
  EChar		Mensaxe[80],Cadena[80];
  HMEM		h;
  ESint32	isSIU;
  EPChar	ClassName;
  EPChar	SubSys;
  EChar         Temp[80];   

  /* We see if the class identifier names a SIU subsystem*/
  strcpy(Temp,ClassNam1);   
  SubSys=strchr(Temp,':');
  
  if (SubSys!=NULL) {
    ClassName=SubSys+1;
    *SubSys=0;
    SubSys=Temp;
    ECout1("New object in system (%s) of class %s",SubSys,ClassName);
    IdC=GetClassId(SubSys);
    if (IdC==-1) {
      ECoutErr ("Class %s doesnt exist",ClassName);
      return -1;
    }
    isSIU=GetProxy(SubSys);
  }  else {
    /* We locate the class */
    ClassName=ClassNam1;
    isSIU=-1;
    IdC=GetClassId(ClassName);
    if (IdC==-1) {
      ECoutErr ("Class %s doesnt exist",ClassName);
      return -1;
    }
    
    if (gClase[IdC].SIUProxy!=0) {
      isSIU=IdC;
      /* if the is in SIU we must create a SIU proxy for the EDMA side*/
      IdC=GVar->SIUProxy[gClase[IdC].SIUProxy].IdClass;
      ECout1 ("New object in SIU subsystem. Using SIU Proxy %s",gClase[IdC].ClassName);
    }
  }
  /* We map the library which creates the object for this process */
  if (ProcMapTable[IdC]!=CLASS_LOADED)
    LoadClassImp(IdC);
  
  /* We get a object identifier*/
  for (i=0;i<GVar->nMaxObj;i++)
    if (gObj[i].Flag==OBJ_FREE)
      break;
  if (i==MAX_OBJ) {
    ECoutErr ("Can't create more objects");
    return -1;
  }
  IdO=i;
  /* We create a data block for this object*/
  
  sprintf (Cadena,"%dData",IdO);
  h=PAlloc(gClase[IdC].TamDatos);
  if (h==(HMEM)0) {
    ECoutErr("Can't create local storage for Object");
    return -1;
  }
  gObj[IdO].SysObj.hDatos=h;
  gObj[IdO].Datos=(EPVoid)PGet(h);
  
  /* Here, the object has benn physically created */
  /* We setup general information for it */
  gObj[IdO].IdObj=IdO;
  gObj[IdO].IdClase=IdC;
  gObj[IdO].IdApp=AppId;
  gObj[IdO].Padre=NULL;
  gObj[IdO].PseudiPadre=NULL;
  gObj[IdO].Flag=OBJ_EXIST;
  gObj[IdO].IdSIU=isSIU;
  gObj[IdO].Final=0;
  /* Modificacin 12 de Noviembre de 1997. Problema en Linux */
  gObj[IdO].VirData.vMet=0;
  gObj[IdO].VirData.vProp=0;
  /* Fin Modificacion */
  
  GVar->nObj++;
  gObj[IdO].nTabla=0;

  /* First we must to alloc memory for virtual methods */
  if (gClase[IdC].nMetVir) {
    gObj[IdO].SysObj.hvTabla=PAlloc(gClase[IdC].nMetVir*sizeof(PMetVir));
    gObj[IdO].vTabla=(PMetVir*)PGet(gObj[IdO].SysObj.hvTabla);
    if (gObj[IdO].SysObj.hvTabla==0) {
      ECoutErr("Can't Allocate Virtual Table");
      return -1;
    }
    j=0;
    for (i=0;i<gClase[IdC].nMet;i++)
      if (pClase[IdC].Met[i].Virtual) {
	gObj[IdO].vTabla[j].Ind=i;
	strcpy ((char*)gObj[IdO].vTabla[j].Id,pClase[IdC].Met[i].IdMet);
	gObj[IdO].vTabla[j].Func=(PPROC)fClase[IdC].Met[i].Func;
	gObj[IdO].vTabla[j].Obj=&gObj[IdO];
	j++;
      }
  }
  
  /* If the object's class is derived... we create subobjects */
  if (gClase[IdC].Derived) {
    /* We build a table for storing subobject information */
    ECoutDbg (4,-1,"(NewObj)Object have %d superclases",gClase[IdC].Derived);
    gObj[IdO].nTabla=gClase[IdC].Derived;
    gObj[IdO].SysObj.hTabla=PAlloc(gClase[IdC].Derived*sizeof(SC_ID));
    gObj[IdO].Tabla=(SC_ID*)PGet(gObj[IdO].SysObj.hTabla);
    if (gObj[IdO].SysObj.hTabla==0) {
      ECoutErr ("Can't create SC table for object");
      return -1;
    }
    for (i=0;i<gClase[IdC].Derived;i++) {
      /*We create the subobject*/
      ECoutDbg (4,-1,"(NewObj)Creating superobject of class %s",gClase[pClase[IdC].SCList[i]].ClassName);
      j=NewObj(gClase[pClase[IdC].SCList[i]].ClassName,ConstPar);
      gObj[IdO].Tabla[i].Obj=&gObj[j];
      strcpy(gObj[IdO].Tabla[i].Id,gClase[pClase[IdC].SCList[i]].ClassName);
      gObj[IdO].Tabla[i].IdClase=pClase[IdC].SCList[i];
      gObj[j].Padre=&gObj[IdO];
      /* Modificacin 10 mayo 1999
       * Como es una superclase, tiene que tener al menos una entrada
       * en la tabla de subclases
       */
      ECoutDbg (4,-1,"(NewObj)Building subclass info. Creating subclass table");

      /* XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
	 FIXME: What if a superclass has other subclases... it'll have
         entries in its subclase table when the subobject gets created.

	 The code here, will destroy the first suclass entry
	 Use AddSC1 instead of the code bellow
	 --------------------------------
	 There is no static inheritance information about subclasses
	 so 0 is the right entry when a new subobject is created
	 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
      */
      gObj[j].nTabla1=1;
      gObj[j].SysObj.hTabla1=PAlloc(1*sizeof(SC_ID));
      gObj[j].Tabla1=(SC_ID*)PGet(gObj[j].SysObj.hTabla1);
      if (gObj[j].SysObj.hTabla1==0) {
	ECoutErr ("Can't create SubClass table for object");
	return -1;
      }
      ECoutDbg(4,-1,"(newObj)Subclass table succesfully created");
      /* We have one entry in the object's subclass table
       * now, we must to fill the entry 0 with the subclass info*/
      gObj[j].Tabla1[0].Obj=&gObj[IdO];


      /*------------------------------*/
      idx=gObj[IdO].IdClase;
      if (pClase[idx].SCIdList[i]) {
	ECout1 ("[NewObj] Preferere Anchor point for %s : %s",
		ClassName,pClase[idx].SCIdList[i]);
	strcpy(gObj[j].Tabla1[0].Id,pClase[idx].SCIdList[i]);
      } else
	strcpy(gObj[j].Tabla1[0].Id,ClassName);
      /*------------------------------*/
      /* FIXME: In the PAP case we'd override the homonym virtual methods
       * in the superclases in order to make the structure consistent*/
      ECoutDbg (4,-1,"(NewObj)ClassName (%s) initiated",ClassName);
      gObj[j].Tabla1[0].IdClase=IdC;
      ECoutDbg(4,-1,"(NewObj)Subclass Info attached");
      /* FIn modificacin 10 de Mayo 1999 */
    }
  }
  /* Here, we execute the constructor */
  for (i=0;i<gClase[IdC].nMet;i++)
    if (strcmp(pClase[IdC].Met[i].IdMet,"born")==0)
      break;
  
  if (i!=gClase[IdC].nMet)	/* if we have got one */
    Met1 (IdO,i,ConstPar);
  
  /* If the object is external using SIU, we invoke NewObj primitive
   * in the SIU proxy object
   */
  if (isSIU!=-1) {
    gObj[IdO].IdSIU=isSIU;
    ECoutDbg (4,-1,"Invoking SIU Proxy NewObj....");
    Met3(IdO,"NewObj",ClassName);
  }
  
  ECoutDbg (4,-1,"(%s) Creates Object %u of Class %s",AppName,IdO,ClassName);
  ECoutDbg (4,-1,"(NewObj)Object of class %s created",ClassName);
  gClase[gObj[IdO].IdClase].Used=1;
  /* User datatypes extension... not fully tested */
  for (i=0;i<gClase[IdC].nProp;i++)
    if (pClase[IdC].Prop[i].Tipo==DT_EUSER) {
      IdC1=pClase[IdC].Prop[i].UserInfo;  /* Get datatype class*/
      j=NewObj(gClase[IdC1].ClassName,NULL); /* Create object*/
      /*We introduce the new object in the right place*/
      *((OBJID*)((EPByte)gObj[IdO].Datos+pClase[IdC].Prop[i].Off))=j;
    }
  return IdO;
}
#endif
/*******************************************************************************/
EUint32 EDMAPROC FreeObj (OBJID IdObj)
{
  EUint32	i,j;
  EUint32	IdClass;
  EChar		Cadena[80];

  if ((IdObj<0)||(IdObj>GVar->nMaxObj)) {
    ECoutErr("(FreeObj) Object %d doesnt exist",IdObj);
    return 0;
  }
  if (gObj[IdObj].Flag==OBJ_FREE) {
    ECoutErr("(FreeObj) Object %d doesnt exist",IdObj);
    return 0;
  }

  IdClass=gObj[IdObj].IdClase;
  /*ECout1 ("[FreeObj]-------------> Freeing object %d of class %s",
    IdObj, gClase[IdClass].ClassName);*/
  
  if (gObj[IdObj].IdSIU!=-1)	/*if we have a SIU proxy*/
    Met3(IdObj,"FreeObj");
  
  /*We look for destructor method*/
  for (i=0;i<gClase[IdClass].nMet;i++)
    if (strcmp(pClase[IdClass].Met[i].IdMet,"rip")==0)
      break;
  
  if (i!=gClase[IdClass].nMet)	/* if we have got one*/
    Met1 (IdObj,i);
  
  /* User datatypes support... not fully tested*/
  for (i=0;i<gClase[IdClass].nProp;i++)
    if (pClase[IdClass].Prop[i].Tipo==DT_EUSER) {
      /*Get the object identifier form object struct*/
      j=*((OBJID*)((EPByte)gObj[IdObj].Datos+pClase[IdClass].Prop[i].Off));
      FreeObj(j); /* and free it */
    }
  
  /* Modificacin temporal 14 de Agosot*/
  GVar->nObj--;
  if (GVar->nObj<0)
    ECout ("(SYSTEM ERROR) Negative number of objects in system...");
  /* FIn modificacion */
  
  /* We decrement object's class ocurrences field */
  gClase[gObj[IdObj].IdClase].Ocurrences--;
  /*
   * Here, we'll test if we've an stock class and we will free
   * class resources id necesary
   */
  
  /* We free subobjects */
  for (i=0;i<gObj[IdObj].nTabla;i++) {
    if (gObj[IdObj].Tabla[i].IdClase!=-1) {
      /*     
      ECout1 ("[FreeObj] Freeing object %d: %s",
	      ((POBJ)gObj[IdObj].Tabla[i].Obj)->IdObj,
	      gClase[((POBJ)gObj[IdObj].Tabla[i].Obj)->IdObj].ClassName);
      */
      FreeObj(((POBJ)gObj[IdObj].Tabla[i].Obj)->IdObj);
    }
    else
      Met3(((POBJ)gObj[IdObj].Tabla[i].Obj)->IdObj,"FreeObj");
  }
  /* We free table of subobjects*/
  if (gObj[IdObj].nTabla)
    PFree (gObj[IdObj].SysObj.hTabla,gObj[IdObj].Tabla);

  gObj[IdObj].nTabla=0;

  /* We free superobjects */
#if 0
  for (i=0;i<gObj[IdObj].nTabla1;i++) {
    if (gObj[IdObj].Tabla1[i].IdClase!=-1)
      FreeObj(((POBJ)gObj[IdObj].Tabla1[i].Obj)->IdObj);
    else
      Met3(((POBJ)gObj[IdObj].Tabla1[i].Obj)->IdObj,"FreeObj");
  }
#endif
  /* We free table of subobjects*/
  if (gObj[IdObj].nTabla1)
    PFree (gObj[IdObj].SysObj.hTabla1,gObj[IdObj].Tabla1);
  
  gObj[IdObj].nTabla1=0;
  gObj[IdObj].Tabla1=NULL;
 
  /*We free data memory block*/
  PFree(gObj[IdObj].SysObj.hDatos,gObj[IdObj].Datos);
  /*We free virtual method information*/
  PFree(gObj[IdObj].SysObj.hvTabla,gObj[IdObj].vTabla);
  gObj[IdObj].vTabla=NULL;
  
  if (gObj[IdObj].Flag==VIRTUAL_OBJECT)
    FreeVirData(&gObj[IdObj],&gObj[IdObj].VirData);
  gObj[IdObj].VirData.vMet=NULL;
  gObj[IdObj].VirData.vProp=NULL;
  
  gObj[IdObj].IdSIU=-1;
  gObj[IdObj].Flag=OBJ_FREE;
  gObj[IdObj].Final=1;
  gObj[IdObj].Padre=NULL;
  gObj[IdObj].PseudiPadre=NULL;
  gObj[IdObj].nTabla=0;
  gObj[IdObj].Tabla=NULL;
  
  ECoutDbg (4,-1,"(%s) Rips Object %u of Class %s",AppName,IdObj,gClase[IdClass].ClassName);
  return 0;
}



EUint32 EDMAPROC FreeOneObj (OBJID IdObj)
{
  EUint32	i,j;
  EUint32	IdClass;
  EChar		Cadena[80];

  if ((IdObj<0)||(IdObj>GVar->nMaxObj)) {
    ECoutErr("(FreeObj) Object %d doesnt exist",IdObj);
    return 0;
  }
  if (gObj[IdObj].Flag==OBJ_FREE) {
    ECoutErr("(FreeObj) Object %d doesnt exist",IdObj);
    return 0;
  }
  
  IdClass=gObj[IdObj].IdClase;
  
  if (gObj[IdObj].IdSIU!=-1)	/*if we have a SIU proxy*/
    Met3(IdObj,"FreeObj");
  
  /*We look for destructor method*/
  for (i=0;i<gClase[IdClass].nMet;i++)
    if (strcmp(pClase[IdClass].Met[i].IdMet,"rip")==0)
      break;
  
  if (i!=gClase[IdClass].nMet)	/* if we have got one*/
    Met1 (IdObj,i);
  
  /* User datatypes support... not fully tested*/
  for (i=0;i<gClase[IdClass].nProp;i++)
    if (pClase[IdClass].Prop[i].Tipo==DT_EUSER) {
      /*Get the object identifier form object struct*/
      j=*((OBJID*)((EPByte)gObj[IdObj].Datos+pClase[IdClass].Prop[i].Off));
      FreeObj(j); /* and free it */
    }
  
  /* Modificacin temporal 14 de Agosot*/
  GVar->nObj--;
  if (GVar->nObj<0)
    ECout ("(SYSTEM ERROR) Negative number of objects in system...");
  /* FIn modificacion */
  
  /* We decrement object's class ocurrences field */
  gClase[gObj[IdObj].IdClase].Ocurrences--;
  /*
   * Here, we'll test if we've an stock class and we will free
   * class resources id necesary
   */
  
  /* We free subobjects. This function leaves  the related object  */
  /*
  for (i=0;i<gObj[IdObj].nTabla;i++) {
    if (gObj[IdObj].Tabla[i].IdClase!=-1)
      FreeObj(((POBJ)gObj[IdObj].Tabla[i].Obj)->IdObj);
    else
      Met3(((POBJ)gObj[IdObj].Tabla[i].Obj)->IdObj,"FreeObj");
  }
  */
  /* We free table of subobjects*/
  PFree (gObj[IdObj].SysObj.hTabla,gObj[IdObj].Tabla);
  
  /*We free data memory block*/
  PFree(gObj[IdObj].SysObj.hDatos,gObj[IdObj].Datos);
  /*We free virtual method information*/
  PFree(gObj[IdObj].SysObj.hvTabla,gObj[IdObj].vTabla);
  gObj[IdObj].vTabla=NULL;
  
  if (gObj[IdObj].Flag==VIRTUAL_OBJECT)
    FreeVirData(&gObj[IdObj],&gObj[IdObj].VirData);
  gObj[IdObj].VirData.vMet=NULL;
  gObj[IdObj].VirData.vProp=NULL;
  
  gObj[IdObj].IdSIU=-1;
  gObj[IdObj].Flag=OBJ_FREE;
  gObj[IdObj].Final=1;
  gObj[IdObj].Padre=NULL;
  gObj[IdObj].PseudiPadre=NULL;
  gObj[IdObj].nTabla=0;
  gObj[IdObj].Tabla=NULL;
  
  ECoutDbg (4,-1,"(%s) Rips Object %u of Class %s",AppName,IdObj,gClase[IdClass].ClassName);
  return 0;
}



/*
** Virtual Objects Support 
*/

EDWord EDMAPROC MakeVirtual (OBJID Objeto)
{
  gObj[Objeto].Flag=VIRTUAL_OBJECT;
  return 0;
}

EUint32 EDMAPROC SetFinal (OBJID IdObj,EUint32 i)
{
  EUint32	old;
  
  old=gObj[IdObj].Final;
  gObj[IdObj].Final=i;
  return old;
}
/* Cast Support FUnctions
 */

/**** UpCastObj
 **   Locates a subobject, up in the inheritance hierarchy that
 **   matchs a class. Does a cast to a more general type
 */
/***********************************************************/
/****** CAUTION!!!!! **************************************/
/****** This code hasn't been tested yet *****************/
OBJID EDMAPROC UpCastObj (OBJID IdObj,EPChar classname) {
  CLASSID      idc;
  ESint32      i,j,ap;
  OBJID        f;
  POBJ         pObj,pObj2;
  
  /* Define start and end point*/
  pObj=&gObj[IdObj];
  idc=GetClassId(classname);
  ap=0;
  /* We must to search through superclasses*/
  if (idc!=-1)  { /* classname is a class name*/
    if (pObj->IdClase==idc)
      return pObj->IdObj;
  }  else /* classname isn't a class name, so condider it an anchor point*/
    ap=1;
    
  for (i=0;i<pObj->nTabla;i++) {
    pObj2=pObj->Tabla[i].Obj;
    if (ap)
      if (strcmp(classname,pObj->Tabla[i].Id)==0) /* We found the anchor point*/
	return pObj2->IdObj;
    /* If we aren't lookgin for an anchor point. Continue looking for class*/
    f=UpCastObj (pObj2->IdObj,classname);
    if (f!=-1)
      return f;	
  }
  return -1;
}

OBJID EDMAPROC DownCastObj (OBJID IdObj,EPChar classname) {
  CLASSID      idc;
  ESint32      i,j,ap;
  OBJID        f;
  POBJ         pObj,pObj2;

  /* Define start and end point*/
  pObj=&gObj[IdObj];
  idc=GetClassId(classname);
  ap=0;
  /* We must to search through superclasses*/
  if (idc!=-1)  { /* classname is a class name*/
    if (pObj->IdClase==idc)
      return pObj->IdObj;
  }  else /* classname isn't a class name, so condider it an anchor point*/
    ap=1;
    
  for (i=0;i<pObj->nTabla1;i++) {
    pObj2=pObj->Tabla1[i].Obj;
    if (ap)
      if (strcmp(classname,pObj->Tabla1[i].Id)==0) /* We found the anchor point*/
	return pObj2->IdObj;
    /* If we aren't lookgin for an anchor point. Continue looking for class*/
    f=UpCastObj (pObj2->IdObj,classname);
    if (f!=-1)
      return f;	
  }
  return -1;
}

/* Simple implementation. First looks up, then looks down*/
OBJID EDMAPROC CastObj (OBJID IdObj,EPChar classname) {
  OBJID id;
  
  id=UpCastObj(IdObj,classname);
  if (id==-1)
    id=DownCastObj(IdObj,classname);
  return id;
}

/* Partial implementation. Don't manage
   - Clonning dynamical classes added to this object
   - Clonning data on static superclasses for this object
   - Clonning dynamical data like EDMAT_BUFFER or string... this is a shallow_clonning
*/
OBJID EDMAPROC CloneObj (OBJID IdObj) {
  OBJID        id_clone;
  CLASSID      idc;
  EChar        classname[1024];
  ESint32      i,j,n;
  EDMAT_BUFFER aux,aux1;

  /* FIXME: Check for boundaries on object identifier*/
  if ((IdObj<0)||(IdObj>GVar->nMaxObj))
    {
      ECoutErr("(CloneObj) Object %d doesnt exist",IdObj);
      return -1;
    }

  idc=gObj[IdObj].IdClase;
  GetClaseName(idc,classname);
  ECout1 ("[DEBUG] CloneObj. Clonning object of class %s",classname);
  /* We create a new object*/
  id_clone=NewObj(classname,NULL);
  if (id_clone==-1) {
    ECoutErr ("Can't clone object %d of class %s",IdObj,classname);
    return -1;
  }
  /* Then we copy raw data block*/
  memcpy (gObj[id_clone].Datos,gObj[IdObj].Datos,gClase[idc].TamDatos);
  /* Now we check for dynamic allocated data. EBUFFER, EZSTRING,*/

  for (i=0;i<gClase[idc].nProp;i++) { 
    if (pClase[idc].Prop[i].nElem>0) {
      ECout1 ("[DEBUG] CloneObj. Located ARRAY type");
    }
    if (pClase[idc].Prop[i].Tipo==DT_EBUFFER) {
      ECout1("[DEBUG] CloneObj. Located EDMAT_BUFFER type");
      /* Clone data*/
      continue;
    }
    if (pClase[idc].Prop[i].Tipo==DT_EZSTRING) {
      ECout1("DEBUG] CloneObj. Located EZSTRING type");
      /* clone data*/
      continue;
    } 

  /* Checks for array types*/
  /* Check for EOBJECT Types*/
  /* Check for EDMAT_BUFFER Types*/
  } 
  /* Initialize vars for superclasses and subclasses related subobjects*/
  return id_clone;
}

OBJID EDMAPROC ShallowCloneObj (OBJID IdObj) {
  ECout1("[INFO] Shallow CloneObj, not yet implemented");
  return -1;
}
/**************************************************************/
/**** Ends CAUTION Note ****************************************/
/*****************************************************************/
/*
** Free virtual object data. Local function
*/
EUint32 FreeVirData (POBJ Obj,VIROBJ_DATA *punt)
{
  PVIR_DATA	temp,temp1;
  HMEM          h;

  /* First we delete method information*/
  temp=Obj->VirData.vMet;
  while (temp!=0) {
    temp1=temp->Next;
    h=temp->h;
    SFree(h,temp);
    temp=temp1;
  }
  /* then properties information */
  temp=Obj->VirData.vProp;
  while (temp!=0) {
    temp1=temp->Next;
    h=temp->h;
    SFree(h,temp);
    temp=temp1;
  }
  
  return 0;
}
