/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
*********************************************************
 Fichero de Cabecera para Portabilidad de tipos
 (c) David Martnez Oliveira
 Vigo, 24 de Septiembre de 1996

 Versin 0.3r1
 REVISIONES:----------------------------------------------------------
 29 de ABril de 1997
 Modificacin para la Beta WIN32. Tenemos conflictos con los nombres
 de tipos. Aadimos la letra 'E' al principio de todas las definiciones
 para evitarlos, esta modificacin afecta a todos los mdulos
 ----------------------------
 21 de Octubre de 1997
 Aadimos tipos para la versin LINUX y compilador GNU
 * -----------------------------------------------------------------------
 * Febraury, 24th, 2001
 * Cleanup code and comment translation
 */
#ifndef PORTABLE_H
#define PORTABLE_H

#ifdef WINAPI_32
#include <windows.h>
#endif

#ifdef __cplusplus
extern "C"{
#endif

#ifdef BC_31
  /* Basic Types */
  typedef	unsigned  char Flag;
  typedef	unsigned  char EByte;
  typedef	unsigned  int  EWord;
  typedef	unsigned  long  EDWord;
  typedef  	unsigned  char EChar;

  /* Integers */
  typedef	unsigned  char 	EUint8;
  typedef 	unsigned  int 	EUint16;
  typedef 	unsigned  long 	EUint32;
  typedef 	signed    char 	ESint8;
  typedef 	signed    int 	ESint16;
  typedef 	signed    long 	ESint32;
  
  /* Reals */
  typedef	float 		EReal32;
  typedef  	double 		EReal64;
  typedef  	long	double 	EReal80;

  /* Pointers */
  typedef   void 		*EPVoid;
  typedef   unsigned  char 	*EPChar;
  typedef   unsigned  char 	*EPByte;
  typedef   unsigned  int 	*EPWord;
  typedef   unsigned  long 	*EPDWord;
  
  typedef   unsigned  char 	*EPUint8;
  typedef   unsigned  int 	*EPUint16;
  typedef   unsigned  long 	*EPUint32;
  typedef   signed    char 	*EPSint8;
  typedef   signed    int 	*EPSint16;
  typedef   signed    long 	*EPSint32;
  
  typedef   float 		*EPReal32;
  typedef   double 		*EPReal64;
  typedef   long      double    *EPReal80;
  
  /* Far Pointers. Used en windows 3.11 version. 
     Will disapear in future versions */
  typedef   void 		far *EPfVoid;
  typedef   unsigned  char 	far *EPfChar;
  typedef   unsigned  char 	far *EPfByte;
  typedef   unsigned  int 	far *EPfWord;
  typedef   unsigned  long 	far *EPfDWord;
  
  typedef   unsigned  char 	far *EPfUint8;
  typedef   unsigned  int 	far *EPfUint16;
  typedef   unsigned  long 	far *EPfUint32;
  typedef   signed    char 	far *EPfSint8;
  typedef   signed    int 	far *EPfSint16;
  typedef   signed    long 	far *EPfSint32;

  typedef   float 		far *EPfReal32;
  typedef   double 		far *EPfReal64;
  typedef   long      double    far *EPfReal80;

#endif

#ifdef EBONY
  /* Basic Types */
  typedef	unsigned  char  Flag;
  typedef	unsigned  char  EByte;
  typedef       unsigned  short EWord;
  typedef	unsigned  long  EDWord;
  typedef  	unsigned  char  EChar;
  
  /* Integer */
  typedef	unsigned  char 	EUint8;
  typedef       unsigned  short EUint16;
  typedef 	unsigned  long 	EUint32;
  typedef 	signed    char 	ESint8;
  typedef       signed    short ESint16;
  typedef 	signed    long 	ESint32;
  
  /* Reals */
  typedef	float 		EReal32;
  typedef  	double 		EReal64;
  typedef  	long	double 	EReal80;
  
  /* Pointers */
  typedef   void 		*EPVoid;
  typedef   char 	        *EPChar;
  typedef   unsigned  char 	*EPByte;
  typedef   unsigned  short     *EPWord;
  typedef   unsigned  long 	*EPDWord;
  
  typedef   unsigned  char 	*EPUint8;
  typedef   unsigned  short     *EPUint16;
  typedef   unsigned  long 	*EPUint32;
  typedef   signed    char 	*EPSint8;
  typedef   signed    short     *EPSint16;
  typedef   signed    long 	*EPSint32;
  
  typedef   float 		*EPReal32;
  typedef   double 		*EPReal64;
  typedef   long      double    *EPReal80;

  /* Far Pointers. Used en windows 3.11 version. 
     Will disapear in future versions */  
  typedef   void               *EPfVoid;
  typedef   unsigned  char     *EPfChar;
  typedef   unsigned  char     *EPfByte;
  typedef   unsigned  short     *EPfWord;
  typedef   unsigned  long     *EPfDWord;
  
  typedef   unsigned  char     *EPfUint8;
  typedef   unsigned  short    *EPfUint16;
  typedef   unsigned  long     *EPfUint32;
  typedef   signed    char     *EPfSint8;
  typedef   signed    short    *EPfSint16;
  typedef   signed    long     *EPfSint32;
  
  typedef   float              *EPfReal32;
  typedef   double             *EPfReal64;
  typedef   long      double   *EPfReal80;
  
#endif

#ifdef GCC
  /* Basic Types */
  typedef	unsigned  char  Flag;
  typedef	unsigned  char  EByte;
  typedef       unsigned  short EWord;
  typedef	unsigned  long  EDWord;
  typedef  	unsigned  char  EChar;
  
  /* Integers */
  typedef	unsigned  char 	EUint8;
  typedef       unsigned  short EUint16;
  typedef 	unsigned  long 	EUint32;
  typedef 	signed    char 	ESint8;
  typedef       signed    short ESint16;
  typedef 	signed    long 	ESint32;
  
  /* Reals */
  typedef	float 		EReal32;
  typedef  	double 		EReal64;
  typedef  	long	double 	EReal80;
  
  /* Pointers */
  typedef   void 		*EPVoid;
  typedef   char 	        *EPChar;
  typedef   unsigned  char 	*EPByte;
  typedef   unsigned  short     *EPWord;
  typedef   unsigned  long 	*EPDWord;
  
  typedef   unsigned  char 	*EPUint8;
  typedef   unsigned  short     *EPUint16;
  typedef   unsigned  long 	*EPUint32;
  typedef   signed    char 	*EPSint8;
  typedef   signed    short     *EPSint16;
  typedef   signed    long 	*EPSint32;
  
  typedef   float 		*EPReal32;
  typedef   double 		*EPReal64;
  typedef   long      double    *EPReal80;

  /* Far Pointers. Used en windows 3.11 version. 
     Will disapear in future versions */  
  typedef   void               *EPfVoid;
  typedef   unsigned  char     *EPfChar;
  typedef   unsigned  char     *EPfByte;
  typedef   unsigned  short    *EPfWord;
  typedef   unsigned  long     *EPfDWord;
  
  typedef   unsigned  char     *EPfUint8;
  typedef   unsigned  short    *EPfUint16;
  typedef   unsigned  long     *EPfUint32;
  typedef   signed    char     *EPfSint8;
  typedef   signed    short    *EPfSint16;
  typedef   signed    long     *EPfSint32;
  
  typedef   float              *EPfReal32;
  typedef   double             *EPfReal64;
  typedef   long      double   *EPfReal80;
  
#endif
  
#ifdef LINUX
#define EDMAPROC
  typedef EUint32 HMEM;
#define TRUE 1
#define FALSE 0   
#endif
  
#ifdef WINAPI_32
  typedef HANDLE	HMEM;
#endif
  
#ifdef WINAPI_32
#define EDMAPROC _export
#endif
  
#ifdef __cplusplus
}
#endif
#endif		
