/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
 Entorno de Desarrollo Modular y Abierto
 (c) David Martnez Oliveira
 Versin Beta 0.3r1
 17 de Octubre de 1997

 Enlace Run-time con EDMA
 
 REVISIONES:-------------------------------------------------------------
 17 de Octubre de 1997
 Aadimos este mdulo a EDMA para el mapeo de los puntos de entrada
 a las primitivas EDMA en las clases.
 *-----------------------------------------------------
 * Febraury, 19th, 2001
 * Code cleanup and comment translation
 * -------------------------------------------------------
 * October, 14th, 2001
 * Added Casting and clonning functions
 * Added EBufferRealloc and PRealloc functions
 * -----------------------------------------------------------
 * October, 19th, 2001
 * Removing warnnings
 * ------------------------------------------------------------
 * November, 11th, 2001
 * Added IsMetAbstract and IsMetStatic
 * ----------------------------------------------------------
 * November, 24th, 2001
 * Added to export IsIDFProxy, IsSIUParser, IsEMIHandler
 * ------------------------------------------------------------
 * November, 25th, 2001
 * Added to export GetNProxies, GetProxyClass
*/

#include <stdio.h>
#include "portable.h"
#include "vglobal.h"
#include "classq.h"
#include "clas.h"
#include "emi.h"
#include "inh.h"
#include "misc.h"
#include "multiidf.h"
#include "siu.h"
#include "obj.h"
#include "objq.h"
#include "poli.h"
#include "pri1.h"
#include "pri3.h"
#include "pri3x.h"
#include "spri3.h"
#include "shmem.h"
#include "vpoli.h"
#include "iniman.h"


EUint32 EDMAPROC BuildProcMap (void)
{
  
  ProcEPTable[0]=(PPROC)GetClaseName;
  ProcEPTable[1]=(PPROC)GetClaseModule;
  ProcEPTable[2]=(PPROC)GetClaseSOId;
  ProcEPTable[3]=(PPROC)GetClaseMaqId;
  ProcEPTable[4]=(PPROC)GetRegClases;
  ProcEPTable[5]=(PPROC)GetClassId;
  
  ProcEPTable[6]=(PPROC)GetPropNum;
  ProcEPTable[7]=(PPROC)GetPropName;
  ProcEPTable[8]=(PPROC)GetPropType;
  ProcEPTable[9]=(PPROC)GetPropId;
  
  ProcEPTable[10]=(PPROC)GetMetNum;
  ProcEPTable[11]=(PPROC)GetMetName;
  ProcEPTable[12]=(PPROC)GetMetSig;
  /*
  ProcEPTable[13]=(PPROC)GetMetIType;
  ProcEPTable[14]=(PPROC)GetMetOType;
  */
  ProcEPTable[15]=(PPROC)IsMetVirtual;
  ProcEPTable[16]=(PPROC)GetMetId;
  
  /* Shared Memory Primitives */
  ProcEPTable[17]=(PPROC)SAlloc;
  ProcEPTable[18]=(PPROC)SFree;
  ProcEPTable[19]=(PPROC)SGet;
  
  /* Private Memory Primitives */
  ProcEPTable[20]=(PPROC)PAlloc;
  ProcEPTable[21]=(PPROC)PFree;
  ProcEPTable[22]=(PPROC)PGet;
  
  ProcEPTable[23]=(PPROC)GetMaqId;
  ProcEPTable[24]=(PPROC)GetSOId;
  ProcEPTable[25]=(PPROC)GetMaqName;
  ProcEPTable[26]=(PPROC)GetSOName;
  ProcEPTable[27]=(PPROC)nSO;
  ProcEPTable[28]=(PPROC)nMaq;
  
  ProcEPTable[29]=(PPROC)NewObj;
  ProcEPTable[30]=(PPROC)FreeObj;
  ProcEPTable[31]=(PPROC)MakeVirtual;
  ProcEPTable[32]=(PPROC)SetFinal;
  
  /* Object query */
  ProcEPTable[33]=(PPROC)GetPObj;
  /*ProcEPTable[34]=GetObjId;*/
  ProcEPTable[35]=(PPROC)GetObjClassName;

  /* Named Objects*/
  ProcEPTable[36]=(PPROC)SetObjName;
  ProcEPTable[37]=(PPROC)GetIdObjbyName;
  ProcEPTable[38]=(PPROC)GetObjName;
  
  ProcEPTable[39]=(PPROC)GetParentId;
  ProcEPTable[40]=(PPROC)GetPseudoParentId;
  /*EPVoid EDMAPROC GetParent (OBJID );*/
  ProcEPTable[41]=(PPROC)GetObjClassId;
  ProcEPTable[42]=(PPROC)GetObjStatus;
  
  ProcEPTable[43]=(PPROC)GetNObj;
  ProcEPTable[44]=(PPROC)GetObjNumSC;
  ProcEPTable[45]=(PPROC)GetObjSC;
  ProcEPTable[46]=(PPROC)GetObjApp;
  ProcEPTable[47]=(PPROC)GetAppId;
  ProcEPTable[48]=(PPROC)ObjReport;
  
  // Level 1 Primitives */
  ProcEPTable[49]=(PPROC)WProp1;
  ProcEPTable[50]=(PPROC)RProp1;
  /*EUint32 EDMAPROC Met1 (OBJID,EUint32,EDWord,EByte);*/
  ProcEPTable[51]=(PPROC)Met1;
  
  
  /* Level 2 Primitives. Now, obsoletes*/
  /*
  ProcEPTable[52]=WProp2;
  ProcEPTable[53]=RProp2;
  ProcEPTable[54]=Met2;
  */
  
  /* Dynamic inheritance */
  ProcEPTable[55]=(PPROC)DeriveClass;
  
  ProcEPTable[56]=(PPROC)AddSCObj;
  ProcEPTable[57]=(PPROC)OverSCObj;
  ProcEPTable[58]=(PPROC)MutateObj;
  
  ProcEPTable[59]=(PPROC)MergeObj;
  ProcEPTable[60]=(PPROC)FreeSC;
  
  /* Level3 Primitives */
  
  ProcEPTable[61]=(PPROC)WProp3;
  
  ProcEPTable[62]=(PPROC)RProp3;
  ProcEPTable[63]=(PPROC)Met3;
  ProcEPTable[64]=(PPROC)Met3S;
  /*EPVoid EDMAPROC GetRef3 (POBJ);*/
  ProcEPTable[65]=(PPROC)GetRef3;
  
  /*
  POBJ EDMAPROC GetSubObj (POBJ ,EPChar ,EPChar ,EPChar);
  POBJ EDMAPROC GetSubObj1 (POBJ,EPChar,EPChar,PVIR_DATA *,EByte,EPChar);
  PVIR_DATA EDMAPROC	GetVirtualData (POBJ,EPChar,PVIR_DATA *,EByte *);
  */

  /* Virtual Methods */
  
  ProcEPTable[66]=(PPROC)OverMet;
  ProcEPTable[67]=(PPROC)OverMet3;
  ProcEPTable[68]=(PPROC)RestMet;
  ProcEPTable[69]=(PPROC)OldMet3;
  
  /* Virtual Objects method override */
  ProcEPTable[70]=(PPROC)VObjOverMet;
  ProcEPTable[71]=(PPROC)VObjOverMet3;
  ProcEPTable[72]=(PPROC)VObjRestMet;
  
  ProcEPTable[73]=(PPROC)VObjOldMet3;
  ProcEPTable[74]=(PPROC)VObjOldMet3V;

  /* Virtual Object Property override */
  ProcEPTable[75]=(PPROC)OverProp;
  ProcEPTable[76]=(PPROC)OverProp3;
  ProcEPTable[77]=(PPROC)RestProp;
  
  ProcEPTable[78]=(PPROC)OldWProp3;
  ProcEPTable[79]=(PPROC)OldWProp3V;
  /*ProcEPTable[80]=OldRProp3;*/
  
  /*
    // Portable threads
    ProcEPTable[81]=EDMAThread;
    ProcEPTable[82]=EDMAExitThread;
    ProcEPTable[83]=EDMAGetThreadSelfId;
    ProcEPTable[84]=EDMAGetThreadSelf;
    ProcEPTable[85]=EDMASetThreadPriority;
    ProcEPTable[86]=EDMAGetThreadPriority;
    
    // Metodos enhebrador
    ProcEPTable[87]=EMet3;
    
    // Notificaciones Asncronas
    ProcEPTable[88]=SetObjNotifyFunc;
    ProcEPTable[89]=DeactivateNotify;
    ProcEPTable[90]=NotStub;
    ProcEPTable[91]=SendNotify;
    
    // Semaforos binarios
    ProcEPTable[92]=CreateBSem;
    ProcEPTable[93]=FreeBSem;
    ProcEPTable[94]=WaitBSem;
    ProcEPTable[95]=SignalBSem;
  */
  /* MIDF: Multi Interface Definition File  */
  ProcEPTable[96]=(PPROC)IDFGetClassID;
  ProcEPTable[97]=(PPROC)IDFSetGeneral;
  ProcEPTable[98]=(PPROC)IDFSetDef;
  ProcEPTable[99]=(PPROC)IDFSetProp;
  ProcEPTable[100]=(PPROC)IDFSetMet;
  ProcEPTable[101]=(PPROC)IDFSetSCList;
  ProcEPTable[102]=(PPROC)IDFSetClassID;
  
  /* Stock Class management*/
  ProcEPTable[103]=(PPROC)AddStockClass;
  ProcEPTable[104]=(PPROC)DelStockClass;
  
  /* EMI Extensions*/
  ProcEPTable[105]=(PPROC)GetEMIComp;
  ProcEPTable[106]=(PPROC)AddEMIComp;
  ProcEPTable[107]=(PPROC)HookGetClass;
  
  /* Miscelanea */
  ProcEPTable[108]=(PPROC)ECout;
  ProcEPTable[109]=(PPROC)SetAppName;
  ProcEPTable[110]=(PPROC)GetSystemPath;
  
  ProcEPTable[111]=(PPROC)SetDebugLevel;
  ProcEPTable[112]=(PPROC)ECoutObj;
  ProcEPTable[113]=(PPROC)ECout1;
  ProcEPTable[114]=(PPROC)ECoutErr;
  ProcEPTable[115]=(PPROC)ECoutDbg;
  
  /* Buffer datatype Management Functions */
  ProcEPTable[116]=(PPROC)EBufferAlloc;
  ProcEPTable[117]=(PPROC)EBufferFree;
  
  ProcEPTable[118]=(PPROC)GetPropTypeId;
  ProcEPTable[119]=(PPROC)GetPropTypeSig;
  ProcEPTable[120]=(PPROC)GetPropNElems;
  ProcEPTable[121]=(PPROC)GetTypeSize;
  ProcEPTable[122]=(PPROC)GetTypeSig;   
  
  ProcEPTable[123]=(PPROC)EDMAWProp3;
  ProcEPTable[124]=(PPROC)EDMARProp3;
  ProcEPTable[125]=(PPROC)EDMAMet3;
  ProcEPTable[126]=(PPROC)EDMAMet3S;
  ProcEPTable[127]=(PPROC)GetPropId3;  
  
  ProcEPTable[128]=(PPROC)LoadClassInt;
  ProcEPTable[129]=(PPROC)UnloadClassInt;
  ProcEPTable[130]=(PPROC)OpenINI;
  ProcEPTable[131]=(PPROC)GetINIString;
  ProcEPTable[132]=(PPROC)GetINIInt;
  ProcEPTable[133]=(PPROC)CloseINI;

  ProcEPTable[134]=(PPROC)SMet3;
  ProcEPTable[135]=(PPROC)SMet3x;
  ProcEPTable[136]=(PPROC)SMet3S;
  ProcEPTable[137]=(PPROC)SMet3Sx;

  ProcEPTable[138]=(PPROC)UpCastObj;
  ProcEPTable[139]=(PPROC)DownCastObj;
  ProcEPTable[140]=(PPROC)CastObj;
  ProcEPTable[141]=(PPROC)CloneObj;
  ProcEPTable[142]=(PPROC)ShallowCloneObj;

  ProcEPTable[143]=(PPROC)EBufferRealloc;
  ProcEPTable[144]=(PPROC)PRealloc;

  ProcEPTable[145]=(PPROC)IsMetAbstract;
  ProcEPTable[146]=(PPROC)IsMetStatic;

  ProcEPTable[147]=(PPROC)GetNParsers;
  ProcEPTable[148]=(PPROC)GetParserClass;

  ProcEPTable[149]=(PPROC)IsIDFParser;
  ProcEPTable[150]=(PPROC)IsSIUProxy;
  ProcEPTable[151]=(PPROC)IsEMIHandler;

  ProcEPTable[152]=(PPROC)GetNProxies;
  ProcEPTable[153]=(PPROC)GetProxyClass;
  ProcEPTable[154]=(PPROC)GetNEMIHandlers;
  ProcEPTable[155]=(PPROC)GetEMIHandlerClass;
  return 0;
}

