/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
*********************************************************
 Entorno de Desarrollo Modular y Abierto
 Versin Beta 0.3r1
 (c) David Martnez Oliveira
 Vigo 15 de Mayo de 1997

 Definicin de tipos Objeto de EDMA
 Revisiones:-------------------------------------------------------------
 23 de Septiembre de 1997
 Aadimos el campo pseudopadre para depuracin	
 -------------------------------------------------------------------------
 4 de Enero de 1998
 Modificamos la estructura OBJ. Aadimos soporte para superclases y 
 subclases, y eliminamos campos que no utilizamos
 * --------------------------------------------------------------
 * Febraury, 20th, 2001
 * Code cleanup and comment translation
 * -----------------------------------------------------------------------
 * December, 8th, 2001
 * Updated PMetVir for virtual method stacking
 *********************************************************
*/
#ifndef TOBJ_H
#define TOBJ_H

#include "esp_typ.h"
#include "ptypes.h"
#ifdef __cplusplus
extern "C"{
#endif
  /*** Data Structs ****************************/

  /* Struts for store objects */
  /* Superclass Identifier struct */
  typedef struct
  {
    EUint32     IdClase;
    EChar	Id[32];		/* Symbolic Identifier for superclass. Upto 32 char. */
    EPVoid	Obj;		/* Pointer to subobject that implements superclass */
  } SC_ID;

#ifndef PTYPES_H
  typedef EChar		IDSIM[32];
#endif

  /* Private Memory Virtual Method struct. For method override*/
  typedef struct PMetVir_t
  {
    EByte       Flag;
    EUint32	Ind;		/* Virtual Method Index in object's vtbl */
    IDSIM	Id[32];		/* Symbolic Identifier for this virtual Method */
    PPROC	Func;		/* Pointer to associated code */
    EPfVoid	Obj;		/* Used for referencing object data from override level */
    struct PMetVir_t *next;     /* For virtual method stacking*/
  } PMetVir;

  /*** Types for Virtual Objects **************************/
  typedef struct	v_dat
  {
    HMEM		h;	  /* Memory Handler for dynamic memory allocation */
    PPROC		Func[4];  /* Up to 4 functions to manage */
    IDSIM		LocalId;  /* Shortcuts for virtual data */
    struct v_dat	*Next;	  /* Next virtual data chunk in list */
    EPfChar		Id;	  /* Symbolic Identifier */
  } VIR_DATA;

  typedef VIR_DATA	*PVIR_DATA;

  typedef struct t_vir_data
  {
    VIR_DATA		*vMet;		/* Virtual Method list */
    VIR_DATA		*vProp;		/* Virtual Properties list */
  } VIROBJ_DATA;
  
  /** Virtual Object data structs ends ******************************/

/*
** TIPOS NOTIFICACIONES **************************************
** NOTA:
**    Al buffer se debera de acceder en exclusin mutua
** Crear funciones portables de objeto exclusin mutua
**    El tamao del buffer debera de ser dinmico
******************************************************************
*/
  /*
typedef struct t_item
	{
    	HMEM		hBlock;
        ESint32		IdObj;
    	HMEM		hBuffer;
    	EPVoid		Buffer;
        EUint32		Size;
	struct t_item	*Next;
        struct t_item	*Prev;
    } NOT_ITEM;

typedef struct
	{
    	EByte		Flag;	// Flag de notificaciones
        PPROC		Func;	// Funcion de notificaciones
        BSEM		Sem;	// Semforo binario de paso
        BSEM		eMutex;	// Para acceso en exclusin mutua a la cola
        NOT_ITEM	*Cola;
        NOT_ITEM	*Cabeza;
    }NOT_DATA;
*/

  /** Level 4 Types. Not used in this implementation***********/
  typedef void	*LEVEL4_DATA;

  /** Shared Object Types *******************************/
  /*
    typedef struct
    {
    BSEM	Mutex;
    } *PSHARE_OBJ;
  */
  /** Level 4 Object Types ends *****************************/
  /*
    4 de Enero de 1999
    --------------------------------------------
    Eliminamos campos de la estructura que no estn siendo utilizados
    Aadimos los campos nTabla1 y Tabla1 para separacin 
    superclases y subclases
  */
  struct	t_obj
  {
    /*	EChar		Name[64];	// Object Name */
    EByte  	Flag;		/* Object Modifier. For the moment marks virtual objects */
    EByte	Final;		/* Does this object allow on-demand inheritance */
    EUint32	IdObj;
    ESint32	IdApp;		/* Application identifier */
    ESint32	IdClase;	/* Object's class Identifier */
    ESint32	IdSIU;		/* SIU Class Identifier */
    SYS_OBJ	SysObj;		/* System Depend data */
    EPfVoid	Datos;          /* Pointer to object's data/properties */
    EUint32	nTabla;		/* Counter for superclass table entries */
    SC_ID  	*Tabla;         /* Superclass table for this object */
    EUint32     nTabla1;        /* Counter for subclass table entries */
    SC_ID       *Tabla1;        /* Subclass Table */
    PMetVir	*vTabla;        /* Virtual Method table */
    VIROBJ_DATA	VirData;	/* Virtual Object Data */
    /*        NOT_DATA	NotData;	// Datos para notificaciones asncronas */
    struct t_obj	*Padre; /* Father object for this one */
    struct t_obj	*PseudiPadre;	/* PseudoFather used for object merging */
    /*        PSHARE_OBJ	Share;		// Informacin de objetos compartidos */
    /*       LEVEL4_DATA	*l4Data;	// Datos para nivel 4 (no implmentado= */
  };

  typedef struct t_obj	OBJ;
  typedef OBJ		*POBJ;
  
  typedef ESint32	OBJID;
  
#ifdef __cplusplus
}
#endif
#endif

