/*
 * EDMA: Entorno de Desarrollo Modular y Abierto
 * Object Oriented and Componetware Framework
 * Copyright (C) 1998-2001  David Martnez Oliveira
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.

 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 
/*
*********************************************************
 Entorno de Desarrollo Modular y Abierto
 (c) David Martnez Oliveira
 Versin Beta 0.3r1

  Vigo 4 de Julio de 1997

 Mdulo para implementacin sobrecarga de mtodos y propiedades
 de objetos virtuales

REVISIONES:--------------------------------------------------
 10 de Julio de 1997
 Test de mtodos virtuales con objetos virtuales satisfactorio.
 Se ha comprobado la funcin en la que se modifica solamente la
 funcin del mtodo, falta probar la que sobreescribe un mtodo con
 otro de otra subclase y las propiedades
 ----------------------
 4 de Agosto
 Todava no cambiamos las funciones de este mdulo para el soporte
 de polimorfismo y multiples parmetros
 -----------------------------
 22 de Agosto de 1997
 Estamos modificando las funciones de objetos virtuales para que funcionen
 en la versin de WIndows 95.
 PROPIEDADES:
 Cuando sobrecargamos una propiedad tenemos que comprobar si existe
 puesto que si estamos sobrecargando una operacin distinta, tendremos que hacerlo
 en la misma entrada de la lista de datos virtuales.

 Por ahora no lo vamos a hacer y no se va a poder sobre cargar
 en lectura
 * -----------------------------------------------------------
 * Febraury, 20th, 2001
 * Code cleanup and comment translation
 * --------------------------------------------------------------
 * October, 19th,2001
 * Removing warnnings
 *********************************************************
*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "portable.h"
#include "classq.h"
#include "shmem.h"
#include "vglobal.h"
#include "clas.h"
#include "obj.h"
#include "pri3.h"
#include "pri1.h"
#include "vpoli.h"
#include "poli.h"

/*** Virtual Objects Primitives **********************************/

/*
***************************************************************
** Implementacin reducida para comprobar mtodo
****************************************************************
** 	- No se realizarn todas las operaciones de comprobacin
**	- Las funciones de sobrecarga:
**		Aaden el elemento indicado a la lista
**		... y lo enlazan apropiadamente
**	- Las operaciones de restauracin
**		Eliminan un elemento de la lista
***************************************************************
*/

EUint32 EDMAPROC VObjOverMet (OBJID IdObj,EPChar Id,PPROC Func)
{
  POBJ		pObj;
  EUint32	nId,i,l,pos,Ind;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		Error[80];
  EPChar	Id1,aux;
  POBJ		OverObj;
  VIR_DATA	*Temp,*Temp1;
  HMEM		h;
  
  pObj=&gObj[IdObj];
  OverObj=pObj;
  if (pObj->Flag!=VIRTUAL_OBJECT)
    {
      ECoutErr ("Object %d isn't virtual",IdObj);
      return 0;
    }
  /* We check if the method exists */
  pObj=GetSubObj (pObj,Id,LocalId,
		  "EDMA Override Method Primitive ERROR");
  
  Tipo=pObj->IdClase;
  for (i=0;i<gClase[Tipo].nMet;i++)
    if (strcmp (pClase[Tipo].Met[i].IdMet,LocalId)==0) break;
  
  Ind=i;
  
  if (gClase[Tipo].nMet==i)
    {
      ECoutErr ("Method %s Not Defined in Class %s",Id,gClase[Tipo].ClassName);
    }
  else
    {
      /* Si el mtodo existe -> Aadimos entrada al objeto */
      h=PAlloc(sizeof(VIR_DATA)+strlen(Id)+1);
      Temp=PGet (h);
      Temp->h=h;
      Temp->Func[0]=Func;
      /* En el atajo ponemos el identificador local */
      aux=Id+strlen(Id);		/* Nos ponemos al final */
      /* Buscamos hacia atrs el separador */
      while ((aux>Id)&&(*aux!='/'))
	aux--;
      strcpy (Temp->LocalId,aux+1);
      memcpy(&Temp->Id,Id,strlen(Id));
      Temp->Next=0;
      /* Localiza el ltimo para enlazar */
      Temp1=OverObj->VirData.vMet;
      if (Temp1==NULL)
	OverObj->VirData.vMet=Temp;
      else
    	{
	  while (Temp1!=0)
	    if (Temp1->Next!=0)
	      Temp1=Temp1->Next;
	    else
	      break;
	  Temp1->Next=Temp;
	}
    }
  return 0;
}

EUint32 EDMAPROC VObjOverMet3 (OBJID IdObj,EPChar Id,EPChar Id1)
{
  POBJ		pObj;
  EUint32	nId,i,l,pos,Ind;
  EUint32	n,j;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		*PosStr;
  EChar		Error[80];
  POBJ		OverObj,Obj1;
  PPROC		Func;
  VIROBJ_DATA	*VirData;
  
  Obj1=pObj;
  Obj1=GetSubObj (Obj1,Id1,LocalId,
		  "EDMA Override Method Primitive ERROR");
  
  Tipo=Obj1->IdClase;
  for (i=0;i<gClase[Tipo].nMet;i++)
    if (strcmp (pClase[Tipo].Met[i].IdMet,LocalId)==0) break;
  
  Ind=i;
  if (gClase[Tipo].nMet==i)
    {
      ECoutErr ("Method %s Not Defined in Class %s",Id,gClase[Tipo].ClassName);
    }
  else
    {
      Func=(PPROC)fClase[Tipo].Met[Ind].Func;
      OverMet(pObj->IdObj,Id,Obj1,Func);
    }
  return 0;
}

EUint32 EDMAPROC VObjRestMet (OBJID IdObj,EPChar Id)
{
  POBJ		pObj;
  EUint32	nId,i,l,pos,Ind;
  EUint32	n,j;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		*PosStr;
  EChar		Error[80];
  EPChar	Id1;
  VIROBJ_DATA	*VirData;
  VIR_DATA	*Temp,*Temp1;
  POBJ		ObjIni;
  EByte		Atajo;
  
  pObj=&gObj[IdObj];
  Temp=0;
  Temp1=pObj->VirData.vMet;
  Temp=GetVirtualData (pObj,Id,&Temp1,&Atajo);

  if (Temp!=0)		/* Si lo encuentra */
    {
      Temp1->Next=Temp->Next;
      if (Temp==pObj->VirData.vMet)	/* SI el que borro es el primero */
	{
	  if (Temp!=Temp1)            /* y no es el nico elemento */
	    pObj->VirData.vMet=Temp1;	/* actualizamos la lista */
	  else
	    pObj->VirData.vMet=0;
	}
      PFree (Temp->h,Temp);
    }

  return 0;
}

EUint32 EDMAPROC VObjOldMet3 (OBJID IdObj,EPChar Id,...)
{
  va_list	p;
  
  va_start(p,Id);
  VObjOldMet3V(IdObj,Id,p);
}

EUint32 EDMAPROC VObjOldMet3V (OBJID IdObj,EPChar Id,EPVoid p)
{
  POBJ		pObj;
  EUint32	nId,i,l,pos,Ind;
  EUint32	n,j;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		*PosStr;
  EChar		Error[80];
  EPChar	Id1;
  VIR_DATA	*VirData;
  VIR_MET_PARAMS VMParams;
  PPROC          Func;   
  
  
  pObj=&gObj[IdObj];
  VirData=0;
  pObj=GetSubObj1 (pObj,Id,LocalId,&VirData,V_MET,
		   "EDMA Run Method Primitive ERROR");
  if (VirData==0)
    {
      Tipo=pObj->IdClase;
      
      for (i=0;i<gClase[Tipo].nMet;i++)
	if (strcmp (pClase[Tipo].Met[i].IdMet,LocalId)==0) break;
      
      Ind=i;
      if (gClase[Tipo].nMet==i)
	{
	  sprintf (Error,"Method %s Not Defined in Class %s",Id,gClase[Tipo].ClassName);
	}
      else
	{
	  if ((pClase[Tipo].Met[Ind].Virtual))
	    {
	      for (i=0;i<gClase[Tipo].nMetVir;i++)
		if (pObj->vTabla[i].Ind==Ind)
		  break;
	      Func=pObj->vTabla[i].Func;
	      pObj=((POBJ)pObj->vTabla[i].Obj);
	    }
	  else
	    Func=(PPROC)fClase[Tipo].Met[Ind].Func;
	  
	  Func (pObj->IdObj,*(EPUint32)p,*((EPUint32)p+1),
		*((EPUint32)p+2),*((EPUint32)p+3),*((EPUint32)p+4),
		*((EPUint32)p+5),*((EPUint32)p+6),*((EPUint32)p+7),
		*((EPUint32)p+8),*((EPUint32)p+9));
	}
    }
  else
    {
      Func=VirData->Func[0];
      VMParams.a=p;
      VMParams.Id=LocalId;
      VMParams.IdPrim=RUN_MET;
      VMParams.IdObj=pObj->IdObj;
      VMParams.Ind=i;
      return Func(pObj,(VIR_MET_PARAMS *)&VMParams);
    }
}

/** SOBRECARGA DE PROPIEDADES *****************************************/
EUint32 EDMAPROC OverProp (OBJID IdObj,EPChar Id,PPROC Func,EByte Flag)
{
  POBJ		pObj;
  EUint32	nId,i,l,pos,Ind;
  EUint32	n,j;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		*PosStr;
  EChar		Error[80];
  EPChar	Id1,aux;
  POBJ		OverObj;
  VIROBJ_DATA	*VirData;
  VIR_DATA	*Temp,*Temp1;
  HMEM		h;
  
  pObj=&gObj[IdObj];
  OverObj=pObj;
  if (pObj->Flag!=VIRTUAL_OBJECT)
    {
      return 0;
    }
  /* Comprobamos si el mtodo existe */
  pObj=GetSubObj (pObj,Id,LocalId,"EDMA Override Property Primitive ERROR");
  
  Tipo=pObj->IdClase;
  for (i=0;i<gClase[Tipo].nProp;i++)
    if (strcmp (pClase[Tipo].Prop[i].IdProp,LocalId)==0) break;
  
  Ind=i;
  if (gClase[Tipo].nProp==i)
    {
      ECoutErr ("Property %s Not Defined in Class %s",Id,gClase[Tipo].ClassName);
    }
  else
    {
      /* Si el mtodo existe -> Aadimos entrada al objeto */
      h=PAlloc(sizeof(VIR_DATA)+strlen(Id)+1);
      Temp=PGet (h);
      Temp->h=h;
      Temp->Func[Flag]=Func;
      /* En el atajo ponemos el identificador local */
      aux=Id+strlen(Id);		/* Nos ponemos al final */
      /* Buscamos hacia atrs el separador */
      while ((aux>Id)&&(*aux!='/'))
	aux--;
      strcpy (Temp->LocalId,aux+1);
      
      memcpy(&Temp->Id,Id,strlen(Id));
      Temp->Next=0;
      /* Localiza el ltimo para enlazar */
      Temp1=OverObj->VirData.vProp;
      if (Temp1==NULL)
	OverObj->VirData.vProp=Temp;
      else
    	{
	  while (Temp1!=0)
	    if (Temp1->Next!=0)
	      Temp1=Temp1->Next;
	    else
	      break;
	  Temp1->Next=Temp;
	}
    }
  
  return 0;
}

EUint32 EDMAPROC OverProp3 (OBJID IdObj,EPChar Id,EPChar Id1,EByte Flag)
{
  POBJ		pObj;
  EUint32	i,Ind;
  EUint32	Tipo;
  EChar		LocalId[50];
  POBJ		OverObj,Obj1;
  PPROC		Func;
  VIROBJ_DATA	*VirData;
  
  pObj=&gObj[IdObj];
  Obj1=pObj;
  Obj1=GetSubObj (Obj1,Id1,LocalId,
		  "EDMA Override Property Primitive ERROR");
  
  Tipo=Obj1->IdClase;
  for (i=0;i<gClase[Tipo].nProp;i++)
    if (strcmp (pClase[Tipo].Prop[i].IdProp,LocalId)==0) break;
  
  Ind=i;
  if (gClase[Tipo].nProp==i)
    {
      ECoutErr ("Property %s Not Defined in Class %s",Id,gClase[Tipo].ClassName);
    }
  else
    {
      Func=(PPROC)fClase[Tipo].Met[Ind].Func;
      OverMet(pObj->IdObj,Id,Obj1,Func);
    }
  return 0;
}



EUint32 EDMAPROC RestProp (OBJID IdObj,EPChar Id,EByte Flag)
{
  POBJ		pObj;
  EUint32	i;
  VIR_DATA	*Temp,*Temp1;
  EByte		Atajo;
  
  pObj=&gObj[IdObj];
  Temp1=pObj->VirData.vProp;
  Temp=GetVirtualData (pObj,Id,&Temp1,&Atajo);

  if (Temp!=0)		/* Si lo encuentra */
    {
      Temp1->Next=Temp->Next;
      if (Temp==pObj->VirData.vProp)	/* SI el que borro es el primero */
	{
	  if (Temp!=Temp1)
	    pObj->VirData.vProp=Temp1;	/* actualizamos la lista */
	  else
	    pObj->VirData.vProp=0;
	}
      PFree (Temp->h,Temp);
    }
  return 0;
}

EUint32 EDMAPROC OldWProp3 (OBJID IdObj,EPChar Id,...)
{
  va_list	p;
  
  va_start(p,Id);
  OldWProp3V(IdObj,Id,p);
}

EUint32 EDMAPROC OldWProp3V (OBJID IdObj,EPChar Id,EPVoid p)
{
  POBJ		pObj;
  EUint32	i;
  EUint32	Tipo;
  EChar		LocalId[50];
  EChar		Error[80];
  VIR_DATA	*VirData;
  VIR_MET_PARAMS VMParams;
  PPROC         Func;
  
  pObj=&gObj[IdObj];
  VirData=0;
  pObj=GetSubObj1 (pObj,Id,LocalId,&VirData,V_PROP,
		   "EDMA Run Method Primitive ERROR");
  if (VirData==0)
    {
      Tipo=pObj->IdClase;
      
      for (i=0;i<gClase[Tipo].nProp;i++)
	if (strcmp (pClase[Tipo].Prop[i].IdProp,LocalId)==0) break;
      
      if (gClase[Tipo].nProp==i)
	{
	  ECoutErr ("Method %s Not Defined in Class %s",Id,gClase[Tipo].ClassName);
	}
      else
    	{
	  EDMAWProp1(pObj->IdObj,i,p);
	  return 0;
	}
    }
  else
    {
      Func=VirData->Func[WRITE_PROP];
      VMParams.a=(EPVoid)p;
      VMParams.Id=LocalId;
      VMParams.IdPrim=WRITE_PROP;
      VMParams.IdObj=pObj->IdObj;
      VMParams.Ind=i;
      return Func(pObj,(VIR_MET_PARAMS *)&VMParams);
      
    }
  
  
}

/*
  EUint32 EDMAPROC OldWProp3 (OBJID IdObj,EPChar Id,EUint32 Val)
  {
  POBJ	pObj;
  
  pObj=&gObj[IdObj];
  OldWProp3S (pObj,Id,Val);
  }
*/
//** ESTAS FUNCIONES NO DEFINIDAS TODAVA
/*
  Uint32 EDMAPROC OldRProp3S (POBJ pObj,PChar Id,Uint32 Val)
  {
  Uint32		nId,i,l,pos;
  Uint32		n,j;
  Uint32		Tipo;
  Char			LocalId[50];
  Char			*PosStr;
  Char			Error[80];
  PfChar		Id1;
  
  //MessageBox (0,"Function Not Implemented","EDMA API ERROR",0);
  return 0;
  pObj=GetSubObj (pObj,Id,LocalId,"EDMA Read Property Primitive ERROR");
  
  Tipo=pObj->IdClase;
  
  for (i=0;i<Clase[Tipo].Prop[i]!=NULL;i++)
  if (strcmp (Clase[Tipo].Prop[i]->IdProp,LocalId)==0) break;
  
  
  if (Clase[Tipo].Prop[i]==NULL)
  {
  sprintf (Error,"Property %s Not Defined in Class %s",Id,Clase[Tipo].NombreClase);
  UserError (Error,"EDMA Read Property Primitive ERROR");
  }
  else
  return (Met1S(pObj,i,Val,1));
  //	return (RProp1S(pObj,i,Val,1));
  
  }

  
  Uint32 EDMAPROC OldRProp3 (Uint32 IdObj,PChar Id,Uint32 Val)
  {
  POBJ	pObj;

//MessageBox (0,"Function Not Implemented","EDMA API ERROR",0);
return 0;

pObj=&Objeto[IdObj];
OldRProp3S (pObj,Id,Val);
}
*/





