/*
    locoos_mixin_test Version 0.0
    Copyright (C) 1998-1999 David Martnez Oliveira
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.    

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <signal.h>
#include <time.h>
#include <unistd.h>

#include <edma.h>

ESint32 mi_puts (OBJID Id,EPChar s,EPSint32 len);
ESint32 mi_gets (OBJID Id,EPChar s,EPSint32 len);

main(int argc,char *argv[])
{
  ESint32          res,len;
  OBJID            id,id1;
  OBJID            id_sc,id_sc1;
  EChar            s[50];

  /* Initialize EDMA System*/
  EDMAInit();
  
  if ((id1 = NewObj ("NFILE1",NULL))==-1) {
    fprintf (stderr,"Can't create object GZFILE...aborting\n");
    EDMAEnd();
    exit(1);
  }

  ObjReport(id1);
  printf ("-----------------------------------\n");
  printf ("Press any key to continue...\n");
  getchar();

  /* Attach TRASP_CODEC using on-demand inheritance */
  Met3 (id1,"CRYPT_BASE>setCodec","TRASP_CODEC");
  /* Then, we override puts and gets with an application function*/
  OverMet (id1,"puts",NULL,(PPROC)mi_puts);
  OverMet (id1,"gets",NULL,(PPROC)mi_gets);

  /* Take a look how the object looks now*/
  ObjReport(id1);
  printf ("-----------------------------------\n");
  printf ("Press any key to continue...\n");
  getchar();

  /* Set the password for our codec, and write a line crypted*/
  Met3 (id1,"SetPassword","mypassword");
  Met3 (id1,"open","crypt_test_file.txt","wt");
  len=20;
  Met3 (id1,"puts","Hello World\n",&len);
  Met3 (id1,"close");

  /* Reopen the file and decrypt de data*/
  Met3 (id1,"open","crypt_test_file","rt");
  len=80;
  Met3 (id1,"gets",&s,&len);
  printf ("[INFO] Read %d bytes: %s\n",len,s);

  printf ("Reading the file with the wrong password");
  Met3 (id1,"SetPassword","nothecorrectone");
  Met3 (id1,"rewind");
  Met3 (id1,"gets",&s,&len);
  printf ("[INFO] Read %d bytes: %s\n",len,s);

  FreeObj(id1);
  /* Shutdown EDMA Suystem */
  EDMAEnd();
  return 0;
}


/************************************************************ 
 * Code for the virtual methods overriden by this application
 * 
 * Note that this code has any dependence with the classes it
 * uses, so you can use this code with any STREAM1 class and any
 * CRYPT_BASE clase, really with any class implementing the methods
 * code and decode... 
 *
 * For example, this code will work with a check speller with
 * a method named code to check texts
 ********************************************************************/
ESint32 mi_puts (OBJID IdObj,EPChar s,EPSint32 len) {
  EDMAT_BUFFER          ibuf,obuf;
  ESint32               r;

  /* Alloc Space... This is a test, so we don't do checks */
  EBufferAlloc (&ibuf,*len);
  EBufferAlloc (&obuf,*len);

  /* Crypt the data to write in the file*/
  memcpy (ibuf.dat,s,*len);
  Met3 (IdObj,"code",ibuf,&obuf,len);
  /* Write it down. We must use the old method*/
  r= (ESint32)OldMet3 (IdObj,"puts",(EPChar)obuf.dat,len);

  /* Free buffers*/
  EBufferFree(&ibuf);
  EBufferFree(&obuf);
  return r;


}
ESint32 mi_gets (OBJID IdObj,EPChar s,EPSint32 len) {
  EDMAT_BUFFER          ibuf,obuf;
  ESint32               r;

  /* Alloc Space... This is a test, so we don't do checks */
  EBufferAlloc (&ibuf,*len);
  EBufferAlloc (&obuf,*len);

  /* Read the data from file using the original NFILE1 method*/
  r= (ESint32)OldMet3 (IdObj,"gets",ibuf.dat,len);
  /* and decrypt it*/
  Met3 (IdObj,"decode",ibuf,&obuf,len);
  strncpy (s,obuf.dat,*len);

  /* Free Buffers*/
  EBufferFree (&ibuf);
  EBufferFree (&obuf);
  return r;
}

